/*
qtractor - An Audio/MIDI multi-track sequencer.

   Copyright (C) 2005-2025, rncbc aka Rui Nuno Capela. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 
*/

/********************************************************************************
** Form generated from reading UI file 'qtractorMidiControlPluginWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QTRACTORMIDICONTROLPLUGINWIDGET_H
#define UI_QTRACTORMIDICONTROLPLUGINWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qtractorMidiControlPluginWidget
{
public:
    QGridLayout *gridLayout;
    QLabel *ControlTypeTextLabel;
    QComboBox *ControlTypeComboBox;
    QSpacerItem *spacerItem;
    QLabel *ControlChannelTextLabel;
    QSpinBox *ControlChannelSpinBox;
    QLabel *ControlParamTextLabel;
    QComboBox *ControlParamComboBox;
    QCheckBox *ControlLogarithmicCheckBox;
    QCheckBox *ControlInvertCheckBox;
    QCheckBox *ControlBipolarCheckBox;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *MidiControlPluginWidget)
    {
        if (MidiControlPluginWidget->objectName().isEmpty())
            MidiControlPluginWidget->setObjectName("MidiControlPluginWidget");
        gridLayout = new QGridLayout(MidiControlPluginWidget);
        gridLayout->setSpacing(4);
        gridLayout->setContentsMargins(4, 4, 4, 4);
        gridLayout->setObjectName("gridLayout");
        ControlTypeTextLabel = new QLabel(MidiControlPluginWidget);
        ControlTypeTextLabel->setObjectName("ControlTypeTextLabel");
        ControlTypeTextLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

        gridLayout->addWidget(ControlTypeTextLabel, 0, 0, 1, 1);

        ControlTypeComboBox = new QComboBox(MidiControlPluginWidget);
        ControlTypeComboBox->setObjectName("ControlTypeComboBox");

        gridLayout->addWidget(ControlTypeComboBox, 0, 1, 1, 1);

        spacerItem = new QSpacerItem(8, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 0, 2, 1, 1);

        ControlChannelTextLabel = new QLabel(MidiControlPluginWidget);
        ControlChannelTextLabel->setObjectName("ControlChannelTextLabel");
        ControlChannelTextLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

        gridLayout->addWidget(ControlChannelTextLabel, 0, 3, 1, 1);

        ControlChannelSpinBox = new QSpinBox(MidiControlPluginWidget);
        ControlChannelSpinBox->setObjectName("ControlChannelSpinBox");
        ControlChannelSpinBox->setAccelerated(false);
        ControlChannelSpinBox->setMinimum(1);
        ControlChannelSpinBox->setMaximum(16);

        gridLayout->addWidget(ControlChannelSpinBox, 0, 4, 1, 1);

        ControlParamTextLabel = new QLabel(MidiControlPluginWidget);
        ControlParamTextLabel->setObjectName("ControlParamTextLabel");
        ControlParamTextLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

        gridLayout->addWidget(ControlParamTextLabel, 1, 0, 1, 1);

        ControlParamComboBox = new QComboBox(MidiControlPluginWidget);
        ControlParamComboBox->setObjectName("ControlParamComboBox");

        gridLayout->addWidget(ControlParamComboBox, 1, 1, 1, 4);

        ControlLogarithmicCheckBox = new QCheckBox(MidiControlPluginWidget);
        ControlLogarithmicCheckBox->setObjectName("ControlLogarithmicCheckBox");

        gridLayout->addWidget(ControlLogarithmicCheckBox, 2, 1, 1, 4);

        ControlInvertCheckBox = new QCheckBox(MidiControlPluginWidget);
        ControlInvertCheckBox->setObjectName("ControlInvertCheckBox");

        gridLayout->addWidget(ControlInvertCheckBox, 3, 1, 1, 4);

        ControlBipolarCheckBox = new QCheckBox(MidiControlPluginWidget);
        ControlBipolarCheckBox->setObjectName("ControlBipolarCheckBox");

        gridLayout->addWidget(ControlBipolarCheckBox, 4, 1, 1, 4);

        spacerItem1 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem1, 0, 5, 5, 1);

#if QT_CONFIG(shortcut)
        ControlTypeTextLabel->setBuddy(ControlTypeComboBox);
        ControlChannelTextLabel->setBuddy(ControlChannelSpinBox);
        ControlParamTextLabel->setBuddy(ControlParamComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(ControlTypeComboBox, ControlChannelSpinBox);
        QWidget::setTabOrder(ControlChannelSpinBox, ControlParamComboBox);
        QWidget::setTabOrder(ControlParamComboBox, ControlLogarithmicCheckBox);
        QWidget::setTabOrder(ControlLogarithmicCheckBox, ControlInvertCheckBox);
        QWidget::setTabOrder(ControlInvertCheckBox, ControlBipolarCheckBox);

        retranslateUi(MidiControlPluginWidget);

        QMetaObject::connectSlotsByName(MidiControlPluginWidget);
    } // setupUi

    void retranslateUi(QWidget *MidiControlPluginWidget)
    {
        ControlTypeTextLabel->setText(QCoreApplication::translate("qtractorMidiControlPluginWidget", "&Type:", nullptr));
#if QT_CONFIG(tooltip)
        ControlTypeComboBox->setToolTip(QCoreApplication::translate("qtractorMidiControlPluginWidget", "MIDI event type", nullptr));
#endif // QT_CONFIG(tooltip)
        ControlChannelTextLabel->setText(QCoreApplication::translate("qtractorMidiControlPluginWidget", "Cha&nnel:", nullptr));
#if QT_CONFIG(tooltip)
        ControlChannelSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiControlPluginWidget", "MIDI channel", nullptr));
#endif // QT_CONFIG(tooltip)
        ControlParamTextLabel->setText(QCoreApplication::translate("qtractorMidiControlPluginWidget", "&Parameter:", nullptr));
#if QT_CONFIG(tooltip)
        ControlParamComboBox->setToolTip(QCoreApplication::translate("qtractorMidiControlPluginWidget", "MIDI parameter", nullptr));
#endif // QT_CONFIG(tooltip)
        ControlLogarithmicCheckBox->setText(QCoreApplication::translate("qtractorMidiControlPluginWidget", "&Logarithmic", nullptr));
        ControlInvertCheckBox->setText(QCoreApplication::translate("qtractorMidiControlPluginWidget", "In&vert", nullptr));
        ControlBipolarCheckBox->setText(QCoreApplication::translate("qtractorMidiControlPluginWidget", "&Bipolar", nullptr));
        (void)MidiControlPluginWidget;
    } // retranslateUi

};

namespace Ui {
    class qtractorMidiControlPluginWidget: public Ui_qtractorMidiControlPluginWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QTRACTORMIDICONTROLPLUGINWIDGET_H
