/****************************************************************************
** Meta object code from reading C++ file 'qtractorMidiSysexForm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorMidiSysexForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorMidiSysexForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21qtractorMidiSysexFormE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorMidiSysexForm::qt_create_metaobjectdata<qt_meta_tag_ZN21qtractorMidiSysexFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorMidiSysexForm",
        "click",
        "",
        "QAbstractButton*",
        "accept",
        "reject",
        "importSlot",
        "exportSlot",
        "moveUpSlot",
        "moveDownSlot",
        "nameChanged",
        "sName",
        "textChanged",
        "openSlot",
        "loadSlot",
        "iName",
        "saveSlot",
        "deleteSlot",
        "addSlot",
        "updateSlot",
        "removeSlot",
        "clearSlot",
        "refreshForm",
        "stabilizeForm"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'click'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Slot 'accept'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'reject'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'importSlot'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'exportSlot'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveUpSlot'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveDownSlot'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'nameChanged'
        QtMocHelpers::SlotData<void(const QString &)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'textChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'openSlot'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'loadSlot'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'saveSlot'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'deleteSlot'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addSlot'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateSlot'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'removeSlot'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'clearSlot'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'refreshForm'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'stabilizeForm'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorMidiSysexForm, qt_meta_tag_ZN21qtractorMidiSysexFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorMidiSysexForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21qtractorMidiSysexFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21qtractorMidiSysexFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21qtractorMidiSysexFormE_t>.metaTypes,
    nullptr
} };

void qtractorMidiSysexForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorMidiSysexForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->click((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 1: _t->accept(); break;
        case 2: _t->reject(); break;
        case 3: _t->importSlot(); break;
        case 4: _t->exportSlot(); break;
        case 5: _t->moveUpSlot(); break;
        case 6: _t->moveDownSlot(); break;
        case 7: _t->nameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->textChanged(); break;
        case 9: _t->openSlot(); break;
        case 10: _t->loadSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->saveSlot(); break;
        case 12: _t->deleteSlot(); break;
        case 13: _t->addSlot(); break;
        case 14: _t->updateSlot(); break;
        case 15: _t->removeSlot(); break;
        case 16: _t->clearSlot(); break;
        case 17: _t->refreshForm(); break;
        case 18: _t->stabilizeForm(); break;
        default: ;
        }
    }
}

const QMetaObject *qtractorMidiSysexForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorMidiSysexForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21qtractorMidiSysexFormE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int qtractorMidiSysexForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
