/*


   Copyright (C) 2005-2025, rncbc aka Rui Nuno Capela. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 
*/

/********************************************************************************
** Form generated from reading UI file 'qtractorAudioIOMatrixForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QTRACTORAUDIOIOMATRIXFORM_H
#define UI_QTRACTORAUDIOIOMATRIXFORM_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QVBoxLayout>
#include "qtractorAudioIOMatrixForm.h"

QT_BEGIN_NAMESPACE

class Ui_qtractorAudioIOMatrixForm
{
public:
    QVBoxLayout *vboxLayout;
    qtractorAudioIOMatrixForm::TableWidget *TableWidget;
    QDialogButtonBox *DialogButtons;

    void setupUi(QDialog *qtractorAudioIOMatrixForm_)
    {
        if (qtractorAudioIOMatrixForm_->objectName().isEmpty())
            qtractorAudioIOMatrixForm_->setObjectName("qtractorAudioIOMatrixForm_");
        qtractorAudioIOMatrixForm_->resize(320, 240);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qtractorAudioIOMatrixForm_->sizePolicy().hasHeightForWidth());
        qtractorAudioIOMatrixForm_->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(qtractorAudioIOMatrixForm_);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        TableWidget = new qtractorAudioIOMatrixForm::TableWidget(qtractorAudioIOMatrixForm_);
        TableWidget->setObjectName("TableWidget");

        vboxLayout->addWidget(TableWidget);

        DialogButtons = new QDialogButtonBox(qtractorAudioIOMatrixForm_);
        DialogButtons->setObjectName("DialogButtons");
        DialogButtons->setOrientation(Qt::Horizontal);
        DialogButtons->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(DialogButtons);

        QWidget::setTabOrder(TableWidget, DialogButtons);

        retranslateUi(qtractorAudioIOMatrixForm_);

        QMetaObject::connectSlotsByName(qtractorAudioIOMatrixForm_);
    } // setupUi

    void retranslateUi(QDialog *qtractorAudioIOMatrixForm_)
    {
        qtractorAudioIOMatrixForm_->setWindowTitle(QCoreApplication::translate("qtractorAudioIOMatrixForm", "Aux-Send I/O Matrix", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qtractorAudioIOMatrixForm: public Ui_qtractorAudioIOMatrixForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QTRACTORAUDIOIOMATRIXFORM_H
