/****************************************************************************
** Meta object code from reading C++ file 'qtractorAudioEngine.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorAudioEngine.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorAudioEngine.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24qtractorAudioEngineProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorAudioEngineProxy::qt_create_metaobjectdata<qt_meta_tag_ZN24qtractorAudioEngineProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorAudioEngineProxy",
        "shutEvent",
        "",
        "xrunEvent",
        "portEvent",
        "buffEvent",
        "iBufferSize",
        "sessEvent",
        "pvSessionArg",
        "syncEvent",
        "iPlayHead",
        "bPlaying",
        "propEvent",
        "selfEvent"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'shutEvent'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'xrunEvent'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'portEvent'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'buffEvent'
        QtMocHelpers::SignalData<void(unsigned int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 6 },
        }}),
        // Signal 'sessEvent'
        QtMocHelpers::SignalData<void(void *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::VoidStar, 8 },
        }}),
        // Signal 'syncEvent'
        QtMocHelpers::SignalData<void(unsigned long, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULong, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Signal 'propEvent'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selfEvent'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorAudioEngineProxy, qt_meta_tag_ZN24qtractorAudioEngineProxyE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorAudioEngineProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24qtractorAudioEngineProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24qtractorAudioEngineProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24qtractorAudioEngineProxyE_t>.metaTypes,
    nullptr
} };

void qtractorAudioEngineProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorAudioEngineProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->shutEvent(); break;
        case 1: _t->xrunEvent(); break;
        case 2: _t->portEvent(); break;
        case 3: _t->buffEvent((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 4: _t->sessEvent((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1]))); break;
        case 5: _t->syncEvent((*reinterpret_cast<std::add_pointer_t<ulong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 6: _t->propEvent(); break;
        case 7: _t->selfEvent(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qtractorAudioEngineProxy::*)()>(_a, &qtractorAudioEngineProxy::shutEvent, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qtractorAudioEngineProxy::*)()>(_a, &qtractorAudioEngineProxy::xrunEvent, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qtractorAudioEngineProxy::*)()>(_a, &qtractorAudioEngineProxy::portEvent, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (qtractorAudioEngineProxy::*)(unsigned int )>(_a, &qtractorAudioEngineProxy::buffEvent, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (qtractorAudioEngineProxy::*)(void * )>(_a, &qtractorAudioEngineProxy::sessEvent, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (qtractorAudioEngineProxy::*)(unsigned long , bool )>(_a, &qtractorAudioEngineProxy::syncEvent, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (qtractorAudioEngineProxy::*)()>(_a, &qtractorAudioEngineProxy::propEvent, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (qtractorAudioEngineProxy::*)()>(_a, &qtractorAudioEngineProxy::selfEvent, 7))
            return;
    }
}

const QMetaObject *qtractorAudioEngineProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorAudioEngineProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24qtractorAudioEngineProxyE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int qtractorAudioEngineProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void qtractorAudioEngineProxy::shutEvent()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void qtractorAudioEngineProxy::xrunEvent()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void qtractorAudioEngineProxy::portEvent()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void qtractorAudioEngineProxy::buffEvent(unsigned int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void qtractorAudioEngineProxy::sessEvent(void * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void qtractorAudioEngineProxy::syncEvent(unsigned long _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void qtractorAudioEngineProxy::propEvent()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void qtractorAudioEngineProxy::selfEvent()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
