/****************************************************************************
** Meta object code from reading C++ file 'qtractorMainForm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorMainForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorMainForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16qtractorMainFormE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorMainForm::qt_create_metaobjectdata<qt_meta_tag_ZN16qtractorMainFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorMainForm",
        "fileNew",
        "",
        "fileOpen",
        "fileOpenRecent",
        "fileSave",
        "fileSaveAs",
        "fileProperties",
        "fileExit",
        "editUndo",
        "editRedo",
        "editCut",
        "editCopy",
        "editPaste",
        "editPasteRepeat",
        "editDelete",
        "editSelectModeClip",
        "editSelectModeRange",
        "editSelectModeRect",
        "editSelectModeCurve",
        "editSelectAll",
        "editSelectNone",
        "editSelectInvert",
        "editSelectTrack",
        "editSelectTrackRange",
        "editSelectRange",
        "editInsertRange",
        "editInsertTrackRange",
        "editRemoveRange",
        "editRemoveTrackRange",
        "editSplit",
        "trackAdd",
        "trackRemove",
        "trackDuplicate",
        "trackProperties",
        "trackInputs",
        "trackOutputs",
        "trackStateRecord",
        "bOn",
        "trackStateMute",
        "trackStateSolo",
        "trackStateMonitor",
        "trackNavigateFirst",
        "trackNavigatePrev",
        "trackNavigateNext",
        "trackNavigateLast",
        "trackNavigateNone",
        "trackMoveTop",
        "trackMoveUp",
        "trackMoveDown",
        "trackMoveBottom",
        "trackHeightIncrease",
        "trackHeightDecrease",
        "trackHeightMinimize",
        "trackHeightReset",
        "trackAutoMonitor",
        "trackAutoDeactivate",
        "trackImportAudio",
        "trackImportMidi",
        "trackExportAudio",
        "trackExportMidi",
        "trackCurveSelect",
        "On",
        "QAction*",
        "pAction",
        "trackCurveMode",
        "trackCurveProcess",
        "trackCurveCapture",
        "trackCurveLocked",
        "trackCurveLogarithmic",
        "trackCurveColor",
        "trackCurveClear",
        "trackCurveProcessAll",
        "trackCurveCaptureAll",
        "trackCurveLockedAll",
        "trackCurveClearAll",
        "clipNew",
        "clipEdit",
        "clipMute",
        "clipUnlink",
        "clipRecordEx",
        "clipSplit",
        "clipMerge",
        "clipNormalize",
        "clipTempoAdjust",
        "clipCrossFade",
        "clipRangeSet",
        "clipLoopSet",
        "clipImport",
        "clipExport",
        "clipToolsQuantize",
        "clipToolsTranspose",
        "clipToolsNormalize",
        "clipToolsRandomize",
        "clipToolsResize",
        "clipToolsRescale",
        "clipToolsTimeshift",
        "clipToolsTemporamp",
        "clipTakeSelect",
        "clipTakeFirst",
        "clipTakePrev",
        "clipTakeNext",
        "clipTakeLast",
        "clipTakeReset",
        "clipTakeRange",
        "viewMenubar",
        "viewStatusbar",
        "viewToolbarFile",
        "viewToolbarEdit",
        "viewToolbarTrack",
        "viewToolbarView",
        "viewToolbarOptions",
        "viewToolbarTransport",
        "viewToolbarTime",
        "viewToolbarThumb",
        "viewToolbarLocked",
        "viewFileSystem",
        "viewFiles",
        "viewMessages",
        "viewConnections",
        "viewMixer",
        "viewZoomIn",
        "viewZoomOut",
        "viewZoomReset",
        "viewZoomHorizontal",
        "viewZoomVertical",
        "viewZoomAll",
        "viewSnapZebra",
        "viewSnapGrid",
        "viewToolTips",
        "viewSnap",
        "viewRefresh",
        "viewInstruments",
        "viewControllers",
        "viewBuses",
        "viewTempoMap",
        "viewOptions",
        "transportBackward",
        "transportRewind",
        "transportFastForward",
        "transportForward",
        "transportStepBackward",
        "transportStepForward",
        "transportLoop",
        "transportLoopSet",
        "transportStop",
        "transportPlay",
        "transportRecord",
        "transportPunch",
        "transportPunchSet",
        "transportCountIn",
        "transportMetro",
        "transportFollow",
        "transportAutoBackward",
        "transportContinue",
        "transportModeNone",
        "transportModeSlave",
        "transportModeMaster",
        "transportModeFull",
        "transportPanic",
        "helpShortcuts",
        "helpAbout",
        "helpAboutQt",
        "stabilizeForm",
        "selectionNotifySlot",
        "qtractorMidiEditor*",
        "pMidiEditor",
        "changeNotifySlot",
        "updateNotifySlot",
        "flags",
        "dirtyNotifySlot",
        "autoSaveAsap",
        "fastTimerSlot",
        "slowTimerSlot",
        "alsaNotify",
        "audioPeakNotify",
        "audioShutNotify",
        "audioXrunNotify",
        "audioPortNotify",
        "audioBuffNotify",
        "iBufferSize",
        "audioSessNotify",
        "pvSessionArg",
        "audioSyncNotify",
        "iPlayHead",
        "bPlaying",
        "audioPropNotify",
        "audioSelfNotify",
        "midiMmcNotify",
        "qtractorMmcEvent",
        "mmce",
        "midiCtlNotify",
        "qtractorCtlEvent",
        "ctle",
        "midiSppNotify",
        "iSppCmd",
        "iSongPos",
        "midiClkNotify",
        "fTempo",
        "midiInpNotify",
        "updateRecentFilesMenu",
        "updateTrackMenu",
        "updateCurveMenu",
        "updateCurveModeMenu",
        "updateClipMenu",
        "updateTakeMenu",
        "updateTakeSelectMenu",
        "updateExportMenu",
        "updateZoomMenu",
        "updateSnapMenu",
        "updateTrackInstrumentMenu",
        "selectAudioFile",
        "sFilename",
        "iTrackChannel",
        "bSelect",
        "activateAudioFile",
        "selectMidiFile",
        "activateMidiFile",
        "activateFile",
        "trackSelectionChanged",
        "mixerSelectionChanged",
        "transportTimeFormatChanged",
        "iDisplayFormat",
        "transportTimeChanged",
        "transportTimeFinished",
        "transportTempoChanged",
        "iBeatsPerBar",
        "iBeatDivisor",
        "transportTempoFinished",
        "snapPerBeatChanged",
        "iSnap",
        "contentsChanged",
        "transportTempoContextMenu",
        "QPoint",
        "pos",
        "handle_sigusr1",
        "handle_sigterm",
        "openNsmSession",
        "saveNsmSession",
        "showNsmSession",
        "hideNsmSession"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'fileNew'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileOpen'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileOpenRecent'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileSave'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileSaveAs'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileProperties'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileExit'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editUndo'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editRedo'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editCut'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editCopy'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editPaste'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editPasteRepeat'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editDelete'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectModeClip'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectModeRange'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectModeRect'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectModeCurve'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectAll'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectNone'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectInvert'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectTrack'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectTrackRange'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSelectRange'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editInsertRange'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editInsertTrackRange'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editRemoveRange'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editRemoveTrackRange'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSplit'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackAdd'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackRemove'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackDuplicate'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackProperties'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackInputs'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackOutputs'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackStateRecord'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackStateMute'
        QtMocHelpers::SlotData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackStateSolo'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackStateMonitor'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackNavigateFirst'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackNavigatePrev'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackNavigateNext'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackNavigateLast'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackNavigateNone'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackMoveTop'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackMoveUp'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackMoveDown'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackMoveBottom'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackHeightIncrease'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackHeightDecrease'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackHeightMinimize'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackHeightReset'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackAutoMonitor'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackAutoDeactivate'
        QtMocHelpers::SlotData<void(bool)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackImportAudio'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackImportMidi'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackExportAudio'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackExportMidi'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackCurveSelect'
        QtMocHelpers::SlotData<void(bool)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 62 },
        }}),
        // Slot 'trackCurveSelect'
        QtMocHelpers::SlotData<void(QAction *, bool)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 63, 64 }, { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackCurveSelect'
        QtMocHelpers::SlotData<void(QAction *)>(61, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 63, 64 },
        }}),
        // Slot 'trackCurveMode'
        QtMocHelpers::SlotData<void(QAction *)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 63, 64 },
        }}),
        // Slot 'trackCurveProcess'
        QtMocHelpers::SlotData<void(bool)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackCurveCapture'
        QtMocHelpers::SlotData<void(bool)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackCurveLocked'
        QtMocHelpers::SlotData<void(bool)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackCurveLogarithmic'
        QtMocHelpers::SlotData<void(bool)>(69, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackCurveColor'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackCurveClear'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'trackCurveProcessAll'
        QtMocHelpers::SlotData<void(bool)>(72, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackCurveCaptureAll'
        QtMocHelpers::SlotData<void(bool)>(73, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackCurveLockedAll'
        QtMocHelpers::SlotData<void(bool)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'trackCurveClearAll'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipNew'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipEdit'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipMute'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipUnlink'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipRecordEx'
        QtMocHelpers::SlotData<void(bool)>(80, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'clipSplit'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipMerge'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipNormalize'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipTempoAdjust'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipCrossFade'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipRangeSet'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipLoopSet'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipImport'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipExport'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipToolsQuantize'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipToolsTranspose'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipToolsNormalize'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipToolsRandomize'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipToolsResize'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipToolsRescale'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipToolsTimeshift'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipToolsTemporamp'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipTakeSelect'
        QtMocHelpers::SlotData<void(QAction *)>(98, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 63, 64 },
        }}),
        // Slot 'clipTakeFirst'
        QtMocHelpers::SlotData<void()>(99, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipTakePrev'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipTakeNext'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipTakeLast'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipTakeReset'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'clipTakeRange'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewMenubar'
        QtMocHelpers::SlotData<void(bool)>(105, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewStatusbar'
        QtMocHelpers::SlotData<void(bool)>(106, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolbarFile'
        QtMocHelpers::SlotData<void(bool)>(107, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolbarEdit'
        QtMocHelpers::SlotData<void(bool)>(108, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolbarTrack'
        QtMocHelpers::SlotData<void(bool)>(109, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolbarView'
        QtMocHelpers::SlotData<void(bool)>(110, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolbarOptions'
        QtMocHelpers::SlotData<void(bool)>(111, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolbarTransport'
        QtMocHelpers::SlotData<void(bool)>(112, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolbarTime'
        QtMocHelpers::SlotData<void(bool)>(113, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolbarThumb'
        QtMocHelpers::SlotData<void(bool)>(114, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolbarLocked'
        QtMocHelpers::SlotData<void(bool)>(115, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewFileSystem'
        QtMocHelpers::SlotData<void(bool)>(116, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewFiles'
        QtMocHelpers::SlotData<void(bool)>(117, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewMessages'
        QtMocHelpers::SlotData<void(bool)>(118, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewConnections'
        QtMocHelpers::SlotData<void(bool)>(119, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewMixer'
        QtMocHelpers::SlotData<void(bool)>(120, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewZoomIn'
        QtMocHelpers::SlotData<void()>(121, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewZoomOut'
        QtMocHelpers::SlotData<void()>(122, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewZoomReset'
        QtMocHelpers::SlotData<void()>(123, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewZoomHorizontal'
        QtMocHelpers::SlotData<void()>(124, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewZoomVertical'
        QtMocHelpers::SlotData<void()>(125, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewZoomAll'
        QtMocHelpers::SlotData<void()>(126, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewSnapZebra'
        QtMocHelpers::SlotData<void(bool)>(127, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewSnapGrid'
        QtMocHelpers::SlotData<void(bool)>(128, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewToolTips'
        QtMocHelpers::SlotData<void(bool)>(129, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'viewSnap'
        QtMocHelpers::SlotData<void()>(130, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewRefresh'
        QtMocHelpers::SlotData<void()>(131, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewInstruments'
        QtMocHelpers::SlotData<void()>(132, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewControllers'
        QtMocHelpers::SlotData<void()>(133, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewBuses'
        QtMocHelpers::SlotData<void()>(134, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewTempoMap'
        QtMocHelpers::SlotData<void()>(135, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewOptions'
        QtMocHelpers::SlotData<void()>(136, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportBackward'
        QtMocHelpers::SlotData<void()>(137, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportRewind'
        QtMocHelpers::SlotData<void()>(138, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportFastForward'
        QtMocHelpers::SlotData<void()>(139, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportForward'
        QtMocHelpers::SlotData<void()>(140, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportStepBackward'
        QtMocHelpers::SlotData<void()>(141, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportStepForward'
        QtMocHelpers::SlotData<void()>(142, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportLoop'
        QtMocHelpers::SlotData<void()>(143, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportLoopSet'
        QtMocHelpers::SlotData<void()>(144, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportStop'
        QtMocHelpers::SlotData<void()>(145, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportPlay'
        QtMocHelpers::SlotData<void()>(146, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportRecord'
        QtMocHelpers::SlotData<void()>(147, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportPunch'
        QtMocHelpers::SlotData<void()>(148, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportPunchSet'
        QtMocHelpers::SlotData<void()>(149, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportCountIn'
        QtMocHelpers::SlotData<void()>(150, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportMetro'
        QtMocHelpers::SlotData<void()>(151, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportFollow'
        QtMocHelpers::SlotData<void()>(152, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportAutoBackward'
        QtMocHelpers::SlotData<void()>(153, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportContinue'
        QtMocHelpers::SlotData<void()>(154, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportModeNone'
        QtMocHelpers::SlotData<void()>(155, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportModeSlave'
        QtMocHelpers::SlotData<void()>(156, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportModeMaster'
        QtMocHelpers::SlotData<void()>(157, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportModeFull'
        QtMocHelpers::SlotData<void()>(158, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'transportPanic'
        QtMocHelpers::SlotData<void()>(159, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'helpShortcuts'
        QtMocHelpers::SlotData<void()>(160, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'helpAbout'
        QtMocHelpers::SlotData<void()>(161, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'helpAboutQt'
        QtMocHelpers::SlotData<void()>(162, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stabilizeForm'
        QtMocHelpers::SlotData<void()>(163, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectionNotifySlot'
        QtMocHelpers::SlotData<void(qtractorMidiEditor *)>(164, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 165, 166 },
        }}),
        // Slot 'changeNotifySlot'
        QtMocHelpers::SlotData<void(qtractorMidiEditor *)>(167, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 165, 166 },
        }}),
        // Slot 'updateNotifySlot'
        QtMocHelpers::SlotData<void(unsigned int)>(168, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 169 },
        }}),
        // Slot 'dirtyNotifySlot'
        QtMocHelpers::SlotData<void()>(170, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'autoSaveAsap'
        QtMocHelpers::SlotData<void()>(171, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fastTimerSlot'
        QtMocHelpers::SlotData<void()>(172, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slowTimerSlot'
        QtMocHelpers::SlotData<void()>(173, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'alsaNotify'
        QtMocHelpers::SlotData<void()>(174, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioPeakNotify'
        QtMocHelpers::SlotData<void()>(175, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioShutNotify'
        QtMocHelpers::SlotData<void()>(176, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioXrunNotify'
        QtMocHelpers::SlotData<void()>(177, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioPortNotify'
        QtMocHelpers::SlotData<void()>(178, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioBuffNotify'
        QtMocHelpers::SlotData<void(unsigned int)>(179, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UInt, 180 },
        }}),
        // Slot 'audioSessNotify'
        QtMocHelpers::SlotData<void(void *)>(181, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::VoidStar, 182 },
        }}),
        // Slot 'audioSyncNotify'
        QtMocHelpers::SlotData<void(unsigned long, bool)>(183, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::ULong, 184 }, { QMetaType::Bool, 185 },
        }}),
        // Slot 'audioPropNotify'
        QtMocHelpers::SlotData<void()>(186, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioSelfNotify'
        QtMocHelpers::SlotData<void()>(187, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiMmcNotify'
        QtMocHelpers::SlotData<void(const qtractorMmcEvent &)>(188, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 189, 190 },
        }}),
        // Slot 'midiCtlNotify'
        QtMocHelpers::SlotData<void(const qtractorCtlEvent &)>(191, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 192, 193 },
        }}),
        // Slot 'midiSppNotify'
        QtMocHelpers::SlotData<void(int, unsigned short)>(194, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 195 }, { QMetaType::UShort, 196 },
        }}),
        // Slot 'midiClkNotify'
        QtMocHelpers::SlotData<void(float)>(197, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Float, 198 },
        }}),
        // Slot 'midiInpNotify'
        QtMocHelpers::SlotData<void(unsigned short)>(199, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UShort, 169 },
        }}),
        // Slot 'updateRecentFilesMenu'
        QtMocHelpers::SlotData<void()>(200, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateTrackMenu'
        QtMocHelpers::SlotData<void()>(201, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateCurveMenu'
        QtMocHelpers::SlotData<void()>(202, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateCurveModeMenu'
        QtMocHelpers::SlotData<void()>(203, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateClipMenu'
        QtMocHelpers::SlotData<void()>(204, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateTakeMenu'
        QtMocHelpers::SlotData<void()>(205, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateTakeSelectMenu'
        QtMocHelpers::SlotData<void()>(206, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateExportMenu'
        QtMocHelpers::SlotData<void()>(207, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateZoomMenu'
        QtMocHelpers::SlotData<void()>(208, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateSnapMenu'
        QtMocHelpers::SlotData<void()>(209, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateTrackInstrumentMenu'
        QtMocHelpers::SlotData<void()>(210, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'selectAudioFile'
        QtMocHelpers::SlotData<void(const QString &, int, bool)>(211, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 212 }, { QMetaType::Int, 213 }, { QMetaType::Bool, 214 },
        }}),
        // Slot 'activateAudioFile'
        QtMocHelpers::SlotData<void(const QString &, int)>(215, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 212 }, { QMetaType::Int, 213 },
        }}),
        // Slot 'activateAudioFile'
        QtMocHelpers::SlotData<void(const QString &)>(215, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 212 },
        }}),
        // Slot 'selectMidiFile'
        QtMocHelpers::SlotData<void(const QString &, int, bool)>(216, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 212 }, { QMetaType::Int, 213 }, { QMetaType::Bool, 214 },
        }}),
        // Slot 'activateMidiFile'
        QtMocHelpers::SlotData<void(const QString &, int)>(217, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 212 }, { QMetaType::Int, 213 },
        }}),
        // Slot 'activateMidiFile'
        QtMocHelpers::SlotData<void(const QString &)>(217, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 212 },
        }}),
        // Slot 'activateFile'
        QtMocHelpers::SlotData<void(const QString &)>(218, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 212 },
        }}),
        // Slot 'trackSelectionChanged'
        QtMocHelpers::SlotData<void()>(219, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'mixerSelectionChanged'
        QtMocHelpers::SlotData<void()>(220, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'transportTimeFormatChanged'
        QtMocHelpers::SlotData<void(int)>(221, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 222 },
        }}),
        // Slot 'transportTimeChanged'
        QtMocHelpers::SlotData<void(unsigned long)>(223, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::ULong, 184 },
        }}),
        // Slot 'transportTimeFinished'
        QtMocHelpers::SlotData<void()>(224, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'transportTempoChanged'
        QtMocHelpers::SlotData<void(float, unsigned short, unsigned short)>(225, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Float, 198 }, { QMetaType::UShort, 226 }, { QMetaType::UShort, 227 },
        }}),
        // Slot 'transportTempoFinished'
        QtMocHelpers::SlotData<void()>(228, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'snapPerBeatChanged'
        QtMocHelpers::SlotData<void(int)>(229, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 230 },
        }}),
        // Slot 'contentsChanged'
        QtMocHelpers::SlotData<void()>(231, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'transportTempoContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(232, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 233, 234 },
        }}),
        // Slot 'handle_sigusr1'
        QtMocHelpers::SlotData<void()>(235, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'handle_sigterm'
        QtMocHelpers::SlotData<void()>(236, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'openNsmSession'
        QtMocHelpers::SlotData<void()>(237, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'saveNsmSession'
        QtMocHelpers::SlotData<void()>(238, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'showNsmSession'
        QtMocHelpers::SlotData<void()>(239, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'hideNsmSession'
        QtMocHelpers::SlotData<void()>(240, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorMainForm, qt_meta_tag_ZN16qtractorMainFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorMainForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16qtractorMainFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16qtractorMainFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16qtractorMainFormE_t>.metaTypes,
    nullptr
} };

void qtractorMainForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorMainForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fileNew(); break;
        case 1: _t->fileOpen(); break;
        case 2: _t->fileOpenRecent(); break;
        case 3: _t->fileSave(); break;
        case 4: _t->fileSaveAs(); break;
        case 5: _t->fileProperties(); break;
        case 6: _t->fileExit(); break;
        case 7: _t->editUndo(); break;
        case 8: _t->editRedo(); break;
        case 9: _t->editCut(); break;
        case 10: _t->editCopy(); break;
        case 11: _t->editPaste(); break;
        case 12: _t->editPasteRepeat(); break;
        case 13: _t->editDelete(); break;
        case 14: _t->editSelectModeClip(); break;
        case 15: _t->editSelectModeRange(); break;
        case 16: _t->editSelectModeRect(); break;
        case 17: _t->editSelectModeCurve(); break;
        case 18: _t->editSelectAll(); break;
        case 19: _t->editSelectNone(); break;
        case 20: _t->editSelectInvert(); break;
        case 21: _t->editSelectTrack(); break;
        case 22: _t->editSelectTrackRange(); break;
        case 23: _t->editSelectRange(); break;
        case 24: _t->editInsertRange(); break;
        case 25: _t->editInsertTrackRange(); break;
        case 26: _t->editRemoveRange(); break;
        case 27: _t->editRemoveTrackRange(); break;
        case 28: _t->editSplit(); break;
        case 29: _t->trackAdd(); break;
        case 30: _t->trackRemove(); break;
        case 31: _t->trackDuplicate(); break;
        case 32: _t->trackProperties(); break;
        case 33: _t->trackInputs(); break;
        case 34: _t->trackOutputs(); break;
        case 35: _t->trackStateRecord((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->trackStateMute((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->trackStateSolo((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->trackStateMonitor((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->trackNavigateFirst(); break;
        case 40: _t->trackNavigatePrev(); break;
        case 41: _t->trackNavigateNext(); break;
        case 42: _t->trackNavigateLast(); break;
        case 43: _t->trackNavigateNone(); break;
        case 44: _t->trackMoveTop(); break;
        case 45: _t->trackMoveUp(); break;
        case 46: _t->trackMoveDown(); break;
        case 47: _t->trackMoveBottom(); break;
        case 48: _t->trackHeightIncrease(); break;
        case 49: _t->trackHeightDecrease(); break;
        case 50: _t->trackHeightMinimize(); break;
        case 51: _t->trackHeightReset(); break;
        case 52: _t->trackAutoMonitor((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 53: _t->trackAutoDeactivate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 54: _t->trackImportAudio(); break;
        case 55: _t->trackImportMidi(); break;
        case 56: _t->trackExportAudio(); break;
        case 57: _t->trackExportMidi(); break;
        case 58: _t->trackCurveSelect((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 59: _t->trackCurveSelect((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 60: _t->trackCurveSelect((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 61: _t->trackCurveMode((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 62: _t->trackCurveProcess((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 63: _t->trackCurveCapture((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 64: _t->trackCurveLocked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 65: _t->trackCurveLogarithmic((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 66: _t->trackCurveColor(); break;
        case 67: _t->trackCurveClear(); break;
        case 68: _t->trackCurveProcessAll((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 69: _t->trackCurveCaptureAll((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 70: _t->trackCurveLockedAll((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 71: _t->trackCurveClearAll(); break;
        case 72: _t->clipNew(); break;
        case 73: _t->clipEdit(); break;
        case 74: _t->clipMute(); break;
        case 75: _t->clipUnlink(); break;
        case 76: _t->clipRecordEx((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 77: _t->clipSplit(); break;
        case 78: _t->clipMerge(); break;
        case 79: _t->clipNormalize(); break;
        case 80: _t->clipTempoAdjust(); break;
        case 81: _t->clipCrossFade(); break;
        case 82: _t->clipRangeSet(); break;
        case 83: _t->clipLoopSet(); break;
        case 84: _t->clipImport(); break;
        case 85: _t->clipExport(); break;
        case 86: _t->clipToolsQuantize(); break;
        case 87: _t->clipToolsTranspose(); break;
        case 88: _t->clipToolsNormalize(); break;
        case 89: _t->clipToolsRandomize(); break;
        case 90: _t->clipToolsResize(); break;
        case 91: _t->clipToolsRescale(); break;
        case 92: _t->clipToolsTimeshift(); break;
        case 93: _t->clipToolsTemporamp(); break;
        case 94: _t->clipTakeSelect((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 95: _t->clipTakeFirst(); break;
        case 96: _t->clipTakePrev(); break;
        case 97: _t->clipTakeNext(); break;
        case 98: _t->clipTakeLast(); break;
        case 99: _t->clipTakeReset(); break;
        case 100: _t->clipTakeRange(); break;
        case 101: _t->viewMenubar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 102: _t->viewStatusbar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 103: _t->viewToolbarFile((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 104: _t->viewToolbarEdit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 105: _t->viewToolbarTrack((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 106: _t->viewToolbarView((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 107: _t->viewToolbarOptions((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 108: _t->viewToolbarTransport((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 109: _t->viewToolbarTime((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 110: _t->viewToolbarThumb((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 111: _t->viewToolbarLocked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 112: _t->viewFileSystem((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 113: _t->viewFiles((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 114: _t->viewMessages((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 115: _t->viewConnections((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 116: _t->viewMixer((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 117: _t->viewZoomIn(); break;
        case 118: _t->viewZoomOut(); break;
        case 119: _t->viewZoomReset(); break;
        case 120: _t->viewZoomHorizontal(); break;
        case 121: _t->viewZoomVertical(); break;
        case 122: _t->viewZoomAll(); break;
        case 123: _t->viewSnapZebra((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 124: _t->viewSnapGrid((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 125: _t->viewToolTips((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 126: _t->viewSnap(); break;
        case 127: _t->viewRefresh(); break;
        case 128: _t->viewInstruments(); break;
        case 129: _t->viewControllers(); break;
        case 130: _t->viewBuses(); break;
        case 131: _t->viewTempoMap(); break;
        case 132: _t->viewOptions(); break;
        case 133: _t->transportBackward(); break;
        case 134: _t->transportRewind(); break;
        case 135: _t->transportFastForward(); break;
        case 136: _t->transportForward(); break;
        case 137: _t->transportStepBackward(); break;
        case 138: _t->transportStepForward(); break;
        case 139: _t->transportLoop(); break;
        case 140: _t->transportLoopSet(); break;
        case 141: _t->transportStop(); break;
        case 142: _t->transportPlay(); break;
        case 143: _t->transportRecord(); break;
        case 144: _t->transportPunch(); break;
        case 145: _t->transportPunchSet(); break;
        case 146: _t->transportCountIn(); break;
        case 147: _t->transportMetro(); break;
        case 148: _t->transportFollow(); break;
        case 149: _t->transportAutoBackward(); break;
        case 150: _t->transportContinue(); break;
        case 151: _t->transportModeNone(); break;
        case 152: _t->transportModeSlave(); break;
        case 153: _t->transportModeMaster(); break;
        case 154: _t->transportModeFull(); break;
        case 155: _t->transportPanic(); break;
        case 156: _t->helpShortcuts(); break;
        case 157: _t->helpAbout(); break;
        case 158: _t->helpAboutQt(); break;
        case 159: _t->stabilizeForm(); break;
        case 160: _t->selectionNotifySlot((*reinterpret_cast<std::add_pointer_t<qtractorMidiEditor*>>(_a[1]))); break;
        case 161: _t->changeNotifySlot((*reinterpret_cast<std::add_pointer_t<qtractorMidiEditor*>>(_a[1]))); break;
        case 162: _t->updateNotifySlot((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 163: _t->dirtyNotifySlot(); break;
        case 164: _t->autoSaveAsap(); break;
        case 165: _t->fastTimerSlot(); break;
        case 166: _t->slowTimerSlot(); break;
        case 167: _t->alsaNotify(); break;
        case 168: _t->audioPeakNotify(); break;
        case 169: _t->audioShutNotify(); break;
        case 170: _t->audioXrunNotify(); break;
        case 171: _t->audioPortNotify(); break;
        case 172: _t->audioBuffNotify((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 173: _t->audioSessNotify((*reinterpret_cast<std::add_pointer_t<void*>>(_a[1]))); break;
        case 174: _t->audioSyncNotify((*reinterpret_cast<std::add_pointer_t<ulong>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 175: _t->audioPropNotify(); break;
        case 176: _t->audioSelfNotify(); break;
        case 177: _t->midiMmcNotify((*reinterpret_cast<std::add_pointer_t<qtractorMmcEvent>>(_a[1]))); break;
        case 178: _t->midiCtlNotify((*reinterpret_cast<std::add_pointer_t<qtractorCtlEvent>>(_a[1]))); break;
        case 179: _t->midiSppNotify((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[2]))); break;
        case 180: _t->midiClkNotify((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 181: _t->midiInpNotify((*reinterpret_cast<std::add_pointer_t<ushort>>(_a[1]))); break;
        case 182: _t->updateRecentFilesMenu(); break;
        case 183: _t->updateTrackMenu(); break;
        case 184: _t->updateCurveMenu(); break;
        case 185: _t->updateCurveModeMenu(); break;
        case 186: _t->updateClipMenu(); break;
        case 187: _t->updateTakeMenu(); break;
        case 188: _t->updateTakeSelectMenu(); break;
        case 189: _t->updateExportMenu(); break;
        case 190: _t->updateZoomMenu(); break;
        case 191: _t->updateSnapMenu(); break;
        case 192: _t->updateTrackInstrumentMenu(); break;
        case 193: _t->selectAudioFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 194: _t->activateAudioFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 195: _t->activateAudioFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 196: _t->selectMidiFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 197: _t->activateMidiFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 198: _t->activateMidiFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 199: _t->activateFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 200: _t->trackSelectionChanged(); break;
        case 201: _t->mixerSelectionChanged(); break;
        case 202: _t->transportTimeFormatChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 203: _t->transportTimeChanged((*reinterpret_cast<std::add_pointer_t<ulong>>(_a[1]))); break;
        case 204: _t->transportTimeFinished(); break;
        case 205: _t->transportTempoChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[3]))); break;
        case 206: _t->transportTempoFinished(); break;
        case 207: _t->snapPerBeatChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 208: _t->contentsChanged(); break;
        case 209: _t->transportTempoContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 210: _t->handle_sigusr1(); break;
        case 211: _t->handle_sigterm(); break;
        case 212: _t->openNsmSession(); break;
        case 213: _t->saveNsmSession(); break;
        case 214: _t->showNsmSession(); break;
        case 215: _t->hideNsmSession(); break;
        default: ;
        }
    }
}

const QMetaObject *qtractorMainForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorMainForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16qtractorMainFormE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int qtractorMainForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 216)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 216;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 216)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 216;
    }
    return _id;
}
QT_WARNING_POP
