/****************************************************************************
** Meta object code from reading C++ file 'qtractorOptionsForm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorOptionsForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorOptionsForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19qtractorOptionsFormE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorOptionsForm::qt_create_metaobjectdata<qt_meta_tag_ZN19qtractorOptionsFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorOptionsForm",
        "accept",
        "",
        "reject",
        "changed",
        "audioCaptureTypeChanged",
        "audioCaptureLatencyChanged",
        "chooseMetroBarFilename",
        "chooseMetroBeatFilename",
        "updateMetroNoteNames",
        "displayFormatChanged",
        "editCustomColorThemes",
        "chooseCustomStyleSheet",
        "chooseCustomIconsTheme",
        "changeAudioMeterLevel",
        "changeMidiMeterLevel",
        "changeAudioMeterColor",
        "changeMidiMeterColor",
        "chooseAudioMeterColor",
        "chooseMidiMeterColor",
        "resetMeterColors",
        "choosePluginType",
        "changePluginPath",
        "choosePluginPath",
        "selectPluginPath",
        "addPluginPath",
        "removePluginPath",
        "moveUpPluginPath",
        "moveDownPluginPath",
        "chooseLv2PresetDir",
        "changePluginBlacklist",
        "choosePluginBlacklist",
        "selectPluginBlacklist",
        "addPluginBlacklist",
        "removePluginBlacklist",
        "clearPluginBlacklist",
        "chooseMessagesFont",
        "chooseMessagesLogPath",
        "chooseSessionTemplatePath",
        "changeOscServer",
        "changeOscActionsItem",
        "QListWidgetItem*",
        "changeOscActions",
        "checkOscActions",
        "uncheckOscActions",
        "resetOscActions",
        "refreshOscActions",
        "stabilizeForm"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'accept'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'reject'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'changed'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioCaptureTypeChanged'
        QtMocHelpers::SlotData<void(int)>(5, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'audioCaptureLatencyChanged'
        QtMocHelpers::SlotData<void(int)>(6, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'chooseMetroBarFilename'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'chooseMetroBeatFilename'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateMetroNoteNames'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'displayFormatChanged'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'editCustomColorThemes'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'chooseCustomStyleSheet'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'chooseCustomIconsTheme'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'changeAudioMeterLevel'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'changeMidiMeterLevel'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'changeAudioMeterColor'
        QtMocHelpers::SlotData<void(const QString &)>(16, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'changeMidiMeterColor'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'chooseAudioMeterColor'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'chooseMidiMeterColor'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'resetMeterColors'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'choosePluginType'
        QtMocHelpers::SlotData<void(int)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'changePluginPath'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'choosePluginPath'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'selectPluginPath'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addPluginPath'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'removePluginPath'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveUpPluginPath'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveDownPluginPath'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'chooseLv2PresetDir'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'changePluginBlacklist'
        QtMocHelpers::SlotData<void(const QString &)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'choosePluginBlacklist'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'selectPluginBlacklist'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addPluginBlacklist'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'removePluginBlacklist'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'clearPluginBlacklist'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'chooseMessagesFont'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'chooseMessagesLogPath'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'chooseSessionTemplatePath'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'changeOscServer'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'changeOscActionsItem'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(40, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 41, 2 },
        }}),
        // Slot 'changeOscActions'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'checkOscActions'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'uncheckOscActions'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'resetOscActions'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'refreshOscActions'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'stabilizeForm'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorOptionsForm, qt_meta_tag_ZN19qtractorOptionsFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorOptionsForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qtractorOptionsFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qtractorOptionsFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19qtractorOptionsFormE_t>.metaTypes,
    nullptr
} };

void qtractorOptionsForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorOptionsForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accept(); break;
        case 1: _t->reject(); break;
        case 2: _t->changed(); break;
        case 3: _t->audioCaptureTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->audioCaptureLatencyChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->chooseMetroBarFilename(); break;
        case 6: _t->chooseMetroBeatFilename(); break;
        case 7: _t->updateMetroNoteNames(); break;
        case 8: _t->displayFormatChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->editCustomColorThemes(); break;
        case 10: _t->chooseCustomStyleSheet(); break;
        case 11: _t->chooseCustomIconsTheme(); break;
        case 12: _t->changeAudioMeterLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 13: _t->changeMidiMeterLevel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->changeAudioMeterColor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->changeMidiMeterColor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->chooseAudioMeterColor(); break;
        case 17: _t->chooseMidiMeterColor(); break;
        case 18: _t->resetMeterColors(); break;
        case 19: _t->choosePluginType((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->changePluginPath((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 21: _t->choosePluginPath(); break;
        case 22: _t->selectPluginPath(); break;
        case 23: _t->addPluginPath(); break;
        case 24: _t->removePluginPath(); break;
        case 25: _t->moveUpPluginPath(); break;
        case 26: _t->moveDownPluginPath(); break;
        case 27: _t->chooseLv2PresetDir(); break;
        case 28: _t->changePluginBlacklist((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->choosePluginBlacklist(); break;
        case 30: _t->selectPluginBlacklist(); break;
        case 31: _t->addPluginBlacklist(); break;
        case 32: _t->removePluginBlacklist(); break;
        case 33: _t->clearPluginBlacklist(); break;
        case 34: _t->chooseMessagesFont(); break;
        case 35: _t->chooseMessagesLogPath(); break;
        case 36: _t->chooseSessionTemplatePath(); break;
        case 37: _t->changeOscServer(); break;
        case 38: _t->changeOscActionsItem((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 39: _t->changeOscActions(); break;
        case 40: _t->checkOscActions(); break;
        case 41: _t->uncheckOscActions(); break;
        case 42: _t->resetOscActions(); break;
        case 43: _t->refreshOscActions(); break;
        case 44: _t->stabilizeForm(); break;
        default: ;
        }
    }
}

const QMetaObject *qtractorOptionsForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorOptionsForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qtractorOptionsFormE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int qtractorOptionsForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 45)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 45)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 45;
    }
    return _id;
}
QT_WARNING_POP
