/*
qtractor - An Audio/MIDI multi-track sequencer.

   Copyright (C) 2005-2025, rncbc aka Rui Nuno Capela. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 
*/

/********************************************************************************
** Form generated from reading UI file 'qtractorMidiToolsForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QTRACTORMIDITOOLSFORM_H
#define UI_QTRACTORMIDITOOLSFORM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qtractorSpinBox.h"

QT_BEGIN_NAMESPACE

class Ui_qtractorMidiToolsForm
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QComboBox *PresetNameComboBox;
    QToolButton *PresetSaveToolButton;
    QToolButton *PresetDeleteToolButton;
    QSpacerItem *spacerItem;
    QTabWidget *ToolTabWidget;
    QWidget *Quantize;
    QGridLayout *gridLayout;
    QCheckBox *QuantizeCheckBox;
    QSpacerItem *spacerItem1;
    QCheckBox *QuantizeTimeCheckBox;
    QComboBox *QuantizeTimeComboBox;
    QDoubleSpinBox *QuantizeTimeSpinBox;
    QSpacerItem *spacerItem2;
    QCheckBox *QuantizeDurationCheckBox;
    QComboBox *QuantizeDurationComboBox;
    QDoubleSpinBox *QuantizeDurationSpinBox;
    QCheckBox *QuantizeSwingCheckBox;
    QComboBox *QuantizeSwingComboBox;
    QDoubleSpinBox *QuantizeSwingSpinBox;
    QComboBox *QuantizeSwingTypeComboBox;
    QSpacerItem *spacerItem3;
    QCheckBox *QuantizeScaleCheckBox;
    QComboBox *QuantizeScaleKeyComboBox;
    QComboBox *QuantizeScaleComboBox;
    QSpacerItem *spacerItem4;
    QWidget *Transpose;
    QGridLayout *gridLayout1;
    QCheckBox *TransposeCheckBox;
    QSpacerItem *spacerItem5;
    QCheckBox *TransposeNoteCheckBox;
    QSpinBox *TransposeNoteSpinBox;
    QSpacerItem *spacerItem6;
    QCheckBox *TransposeTimeCheckBox;
    qtractorTimeSpinBox *TransposeTimeSpinBox;
    QComboBox *TransposeFormatComboBox;
    QSpacerItem *spacerItem7;
    QCheckBox *TransposeReverseCheckBox;
    QSpacerItem *spacerItem8;
    QWidget *Normalize;
    QGridLayout *gridLayout2;
    QCheckBox *NormalizeCheckBox;
    QSpacerItem *spacerItem9;
    QCheckBox *NormalizePercentCheckBox;
    QDoubleSpinBox *NormalizePercentSpinBox;
    QSpacerItem *spacerItem10;
    QCheckBox *NormalizeValueCheckBox;
    QSpinBox *NormalizeValueSpinBox;
    QCheckBox *NormalizeCompressCheckBox;
    QSpacerItem *spacerItem11;
    QWidget *Randomize;
    QGridLayout *gridLayout3;
    QCheckBox *RandomizeCheckBox;
    QSpacerItem *spacerItem12;
    QCheckBox *RandomizeNoteCheckBox;
    QDoubleSpinBox *RandomizeNoteSpinBox;
    QSpacerItem *spacerItem13;
    QCheckBox *RandomizeTimeCheckBox;
    QDoubleSpinBox *RandomizeTimeSpinBox;
    QCheckBox *RandomizeDurationCheckBox;
    QDoubleSpinBox *RandomizeDurationSpinBox;
    QCheckBox *RandomizeValueCheckBox;
    QDoubleSpinBox *RandomizeValueSpinBox;
    QSpacerItem *spacerItem14;
    QWidget *Resize;
    QGridLayout *gridLayout4;
    QCheckBox *ResizeCheckBox;
    QSpacerItem *spacerItem15;
    QCheckBox *ResizeDurationCheckBox;
    qtractorTimeSpinBox *ResizeDurationSpinBox;
    QComboBox *ResizeDurationFormatComboBox;
    QSpacerItem *spacerItem16;
    QCheckBox *ResizeValueCheckBox;
    QSpinBox *ResizeValueSpinBox;
    QComboBox *ResizeValue2ComboBox;
    QSpinBox *ResizeValue2SpinBox;
    QSpacerItem *spacerItem17;
    QCheckBox *ResizeLegatoCheckBox;
    QComboBox *ResizeLegatoTypeComboBox;
    QComboBox *ResizeLegatoLengthComboBox;
    QComboBox *ResizeLegatoModeComboBox;
    QSpacerItem *spacerItem18;
    QHBoxLayout *hboxLayout1;
    QCheckBox *ResizeJoinCheckBox;
    QCheckBox *ResizeSplitCheckBox;
    QComboBox *ResizeSplitLengthComboBox;
    QComboBox *ResizeSplitOffsetComboBox;
    QSpacerItem *spacerItem19;
    QSpacerItem *spacerItem20;
    QWidget *Rescale;
    QGridLayout *gridLayout5;
    QCheckBox *RescaleCheckBox;
    QSpacerItem *spacerItem21;
    QCheckBox *RescaleTimeCheckBox;
    QDoubleSpinBox *RescaleTimeSpinBox;
    QSpacerItem *spacerItem22;
    QCheckBox *RescaleDurationCheckBox;
    QDoubleSpinBox *RescaleDurationSpinBox;
    QCheckBox *RescaleValueCheckBox;
    QDoubleSpinBox *RescaleValueSpinBox;
    QCheckBox *RescaleInvertCheckBox;
    QSpacerItem *spacerItem23;
    QWidget *Timeshift;
    QGridLayout *gridLayout6;
    QCheckBox *TimeshiftCheckBox;
    QSpacerItem *spacerItem24;
    QLabel *TimeshiftLabel;
    QDoubleSpinBox *TimeshiftSpinBox;
    QSlider *TimeshiftSlider;
    QFrame *TimeshiftFrame;
    QLabel *TimeshiftText;
    QCheckBox *TimeshiftDurationCheckBox;
    QSpacerItem *spacerItem25;
    QWidget *Temporamp;
    QGridLayout *gridLayout7;
    QCheckBox *TemporampCheckBox;
    QSpacerItem *spacerItem26;
    QLabel *TemporampFromLabel;
    qtractorTempoSpinBox *TemporampFromSpinBox;
    QLabel *TemporampToLabel;
    qtractorTempoSpinBox *TemporampToSpinBox;
    QSpacerItem *spacerItem27;
    QLabel *TemporampText;
    QCheckBox *TemporampDurationCheckBox;
    QSpacerItem *spacerItem28;
    QDialogButtonBox *DialogButtonBox;

    void setupUi(QDialog *qtractorMidiToolsForm)
    {
        if (qtractorMidiToolsForm->objectName().isEmpty())
            qtractorMidiToolsForm->setObjectName("qtractorMidiToolsForm");
        qtractorMidiToolsForm->resize(480, 240);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("qtractorMidiEditor")));
        qtractorMidiToolsForm->setWindowIcon(icon);
        vboxLayout = new QVBoxLayout(qtractorMidiToolsForm);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(2);
        hboxLayout->setContentsMargins(2, 2, 2, 2);
        hboxLayout->setObjectName("hboxLayout");
        PresetNameComboBox = new QComboBox(qtractorMidiToolsForm);
        PresetNameComboBox->setObjectName("PresetNameComboBox");
        PresetNameComboBox->setMinimumSize(QSize(160, 0));
        PresetNameComboBox->setEditable(true);

        hboxLayout->addWidget(PresetNameComboBox);

        PresetSaveToolButton = new QToolButton(qtractorMidiToolsForm);
        PresetSaveToolButton->setObjectName("PresetSaveToolButton");
        PresetSaveToolButton->setFocusPolicy(Qt::TabFocus);
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("formSave")));
        PresetSaveToolButton->setIcon(icon1);

        hboxLayout->addWidget(PresetSaveToolButton);

        PresetDeleteToolButton = new QToolButton(qtractorMidiToolsForm);
        PresetDeleteToolButton->setObjectName("PresetDeleteToolButton");
        PresetDeleteToolButton->setFocusPolicy(Qt::TabFocus);
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("formRemove")));
        PresetDeleteToolButton->setIcon(icon2);

        hboxLayout->addWidget(PresetDeleteToolButton);

        spacerItem = new QSpacerItem(180, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);


        vboxLayout->addLayout(hboxLayout);

        ToolTabWidget = new QTabWidget(qtractorMidiToolsForm);
        ToolTabWidget->setObjectName("ToolTabWidget");
        Quantize = new QWidget();
        Quantize->setObjectName("Quantize");
        gridLayout = new QGridLayout(Quantize);
        gridLayout->setSpacing(4);
        gridLayout->setContentsMargins(4, 4, 4, 4);
        gridLayout->setObjectName("gridLayout");
        QuantizeCheckBox = new QCheckBox(Quantize);
        QuantizeCheckBox->setObjectName("QuantizeCheckBox");
        QFont font;
        font.setBold(true);
        QuantizeCheckBox->setFont(font);

        gridLayout->addWidget(QuantizeCheckBox, 0, 0, 1, 6);

        spacerItem1 = new QSpacerItem(20, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem1, 1, 0, 1, 1);

        QuantizeTimeCheckBox = new QCheckBox(Quantize);
        QuantizeTimeCheckBox->setObjectName("QuantizeTimeCheckBox");

        gridLayout->addWidget(QuantizeTimeCheckBox, 1, 1, 1, 1);

        QuantizeTimeComboBox = new QComboBox(Quantize);
        QuantizeTimeComboBox->setObjectName("QuantizeTimeComboBox");

        gridLayout->addWidget(QuantizeTimeComboBox, 1, 2, 1, 1);

        QuantizeTimeSpinBox = new QDoubleSpinBox(Quantize);
        QuantizeTimeSpinBox->setObjectName("QuantizeTimeSpinBox");
        QuantizeTimeSpinBox->setMinimumSize(QSize(60, 0));
        QuantizeTimeSpinBox->setAccelerated(true);
        QuantizeTimeSpinBox->setDecimals(1);
        QuantizeTimeSpinBox->setMinimum(0.000000000000000);
        QuantizeTimeSpinBox->setMaximum(100.000000000000000);
        QuantizeTimeSpinBox->setSingleStep(0.100000000000000);
        QuantizeTimeSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(QuantizeTimeSpinBox, 1, 3, 1, 1);

        spacerItem2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem2, 1, 4, 2, 2);

        QuantizeDurationCheckBox = new QCheckBox(Quantize);
        QuantizeDurationCheckBox->setObjectName("QuantizeDurationCheckBox");

        gridLayout->addWidget(QuantizeDurationCheckBox, 2, 1, 1, 1);

        QuantizeDurationComboBox = new QComboBox(Quantize);
        QuantizeDurationComboBox->setObjectName("QuantizeDurationComboBox");

        gridLayout->addWidget(QuantizeDurationComboBox, 2, 2, 1, 1);

        QuantizeDurationSpinBox = new QDoubleSpinBox(Quantize);
        QuantizeDurationSpinBox->setObjectName("QuantizeDurationSpinBox");
        QuantizeDurationSpinBox->setMinimumSize(QSize(60, 0));
        QuantizeDurationSpinBox->setAccelerated(true);
        QuantizeDurationSpinBox->setDecimals(1);
        QuantizeDurationSpinBox->setMinimum(0.000000000000000);
        QuantizeDurationSpinBox->setMaximum(100.000000000000000);
        QuantizeDurationSpinBox->setSingleStep(0.100000000000000);
        QuantizeDurationSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(QuantizeDurationSpinBox, 2, 3, 1, 1);

        QuantizeSwingCheckBox = new QCheckBox(Quantize);
        QuantizeSwingCheckBox->setObjectName("QuantizeSwingCheckBox");

        gridLayout->addWidget(QuantizeSwingCheckBox, 3, 1, 1, 1);

        QuantizeSwingComboBox = new QComboBox(Quantize);
        QuantizeSwingComboBox->setObjectName("QuantizeSwingComboBox");

        gridLayout->addWidget(QuantizeSwingComboBox, 3, 2, 1, 1);

        QuantizeSwingSpinBox = new QDoubleSpinBox(Quantize);
        QuantizeSwingSpinBox->setObjectName("QuantizeSwingSpinBox");
        QuantizeSwingSpinBox->setMinimumSize(QSize(60, 0));
        QuantizeSwingSpinBox->setAccelerated(true);
        QuantizeSwingSpinBox->setDecimals(1);
        QuantizeSwingSpinBox->setMinimum(-100.000000000000000);
        QuantizeSwingSpinBox->setMaximum(100.000000000000000);
        QuantizeSwingSpinBox->setSingleStep(0.100000000000000);
        QuantizeSwingSpinBox->setValue(0.000000000000000);

        gridLayout->addWidget(QuantizeSwingSpinBox, 3, 3, 1, 1);

        QuantizeSwingTypeComboBox = new QComboBox(Quantize);
        QuantizeSwingTypeComboBox->addItem(QString());
        QuantizeSwingTypeComboBox->addItem(QString());
        QuantizeSwingTypeComboBox->addItem(QString());
        QuantizeSwingTypeComboBox->setObjectName("QuantizeSwingTypeComboBox");

        gridLayout->addWidget(QuantizeSwingTypeComboBox, 3, 4, 1, 1);

        spacerItem3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem3, 3, 5, 1, 1);

        QuantizeScaleCheckBox = new QCheckBox(Quantize);
        QuantizeScaleCheckBox->setObjectName("QuantizeScaleCheckBox");

        gridLayout->addWidget(QuantizeScaleCheckBox, 4, 1, 1, 1);

        QuantizeScaleKeyComboBox = new QComboBox(Quantize);
        QuantizeScaleKeyComboBox->setObjectName("QuantizeScaleKeyComboBox");

        gridLayout->addWidget(QuantizeScaleKeyComboBox, 4, 2, 1, 1);

        QuantizeScaleComboBox = new QComboBox(Quantize);
        QuantizeScaleComboBox->setObjectName("QuantizeScaleComboBox");

        gridLayout->addWidget(QuantizeScaleComboBox, 4, 3, 1, 2);

        spacerItem4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem4, 5, 1, 1, 5);

        ToolTabWidget->addTab(Quantize, QString());
        Transpose = new QWidget();
        Transpose->setObjectName("Transpose");
        gridLayout1 = new QGridLayout(Transpose);
        gridLayout1->setSpacing(4);
        gridLayout1->setContentsMargins(4, 4, 4, 4);
        gridLayout1->setObjectName("gridLayout1");
        TransposeCheckBox = new QCheckBox(Transpose);
        TransposeCheckBox->setObjectName("TransposeCheckBox");
        TransposeCheckBox->setFont(font);

        gridLayout1->addWidget(TransposeCheckBox, 0, 0, 1, 6);

        spacerItem5 = new QSpacerItem(16, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacerItem5, 1, 0, 1, 1);

        TransposeNoteCheckBox = new QCheckBox(Transpose);
        TransposeNoteCheckBox->setObjectName("TransposeNoteCheckBox");

        gridLayout1->addWidget(TransposeNoteCheckBox, 1, 1, 1, 1);

        TransposeNoteSpinBox = new QSpinBox(Transpose);
        TransposeNoteSpinBox->setObjectName("TransposeNoteSpinBox");
        TransposeNoteSpinBox->setMinimumSize(QSize(60, 0));
        TransposeNoteSpinBox->setMinimum(-64);
        TransposeNoteSpinBox->setMaximum(64);

        gridLayout1->addWidget(TransposeNoteSpinBox, 1, 2, 1, 1);

        spacerItem6 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacerItem6, 1, 3, 1, 3);

        TransposeTimeCheckBox = new QCheckBox(Transpose);
        TransposeTimeCheckBox->setObjectName("TransposeTimeCheckBox");

        gridLayout1->addWidget(TransposeTimeCheckBox, 2, 1, 1, 1);

        TransposeTimeSpinBox = new qtractorTimeSpinBox(Transpose);
        TransposeTimeSpinBox->setObjectName("TransposeTimeSpinBox");
        TransposeTimeSpinBox->setMinimumSize(QSize(120, 0));

        gridLayout1->addWidget(TransposeTimeSpinBox, 2, 2, 1, 2);

        TransposeFormatComboBox = new QComboBox(Transpose);
        TransposeFormatComboBox->addItem(QString());
        TransposeFormatComboBox->addItem(QString());
        TransposeFormatComboBox->addItem(QString());
        TransposeFormatComboBox->setObjectName("TransposeFormatComboBox");

        gridLayout1->addWidget(TransposeFormatComboBox, 2, 4, 1, 1);

        spacerItem7 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacerItem7, 2, 5, 1, 1);

        TransposeReverseCheckBox = new QCheckBox(Transpose);
        TransposeReverseCheckBox->setObjectName("TransposeReverseCheckBox");

        gridLayout1->addWidget(TransposeReverseCheckBox, 3, 2, 1, 2);

        spacerItem8 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout1->addItem(spacerItem8, 4, 1, 1, 5);

        ToolTabWidget->addTab(Transpose, QString());
        Normalize = new QWidget();
        Normalize->setObjectName("Normalize");
        gridLayout2 = new QGridLayout(Normalize);
        gridLayout2->setSpacing(4);
        gridLayout2->setContentsMargins(4, 4, 4, 4);
        gridLayout2->setObjectName("gridLayout2");
        NormalizeCheckBox = new QCheckBox(Normalize);
        NormalizeCheckBox->setObjectName("NormalizeCheckBox");
        NormalizeCheckBox->setFont(font);

        gridLayout2->addWidget(NormalizeCheckBox, 0, 0, 1, 4);

        spacerItem9 = new QSpacerItem(16, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(spacerItem9, 1, 0, 1, 1);

        NormalizePercentCheckBox = new QCheckBox(Normalize);
        NormalizePercentCheckBox->setObjectName("NormalizePercentCheckBox");

        gridLayout2->addWidget(NormalizePercentCheckBox, 1, 1, 1, 1);

        NormalizePercentSpinBox = new QDoubleSpinBox(Normalize);
        NormalizePercentSpinBox->setObjectName("NormalizePercentSpinBox");
        NormalizePercentSpinBox->setMinimumSize(QSize(60, 0));
        NormalizePercentSpinBox->setAccelerated(true);
        NormalizePercentSpinBox->setDecimals(1);
        NormalizePercentSpinBox->setMaximum(150.000000000000000);
        NormalizePercentSpinBox->setSingleStep(0.100000000000000);

        gridLayout2->addWidget(NormalizePercentSpinBox, 1, 2, 1, 1);

        spacerItem10 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(spacerItem10, 1, 3, 2, 1);

        NormalizeValueCheckBox = new QCheckBox(Normalize);
        NormalizeValueCheckBox->setObjectName("NormalizeValueCheckBox");

        gridLayout2->addWidget(NormalizeValueCheckBox, 2, 1, 1, 1);

        NormalizeValueSpinBox = new QSpinBox(Normalize);
        NormalizeValueSpinBox->setObjectName("NormalizeValueSpinBox");
        NormalizeValueSpinBox->setMinimumSize(QSize(60, 0));
        NormalizeValueSpinBox->setMaximum(127);

        gridLayout2->addWidget(NormalizeValueSpinBox, 2, 2, 1, 1);

        NormalizeCompressCheckBox = new QCheckBox(Normalize);
        NormalizeCompressCheckBox->setObjectName("NormalizeCompressCheckBox");

        gridLayout2->addWidget(NormalizeCompressCheckBox, 3, 1, 1, 1);

        spacerItem11 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout2->addItem(spacerItem11, 4, 1, 1, 3);

        ToolTabWidget->addTab(Normalize, QString());
        Randomize = new QWidget();
        Randomize->setObjectName("Randomize");
        gridLayout3 = new QGridLayout(Randomize);
        gridLayout3->setSpacing(4);
        gridLayout3->setContentsMargins(4, 4, 4, 4);
        gridLayout3->setObjectName("gridLayout3");
        RandomizeCheckBox = new QCheckBox(Randomize);
        RandomizeCheckBox->setObjectName("RandomizeCheckBox");
        RandomizeCheckBox->setFont(font);

        gridLayout3->addWidget(RandomizeCheckBox, 0, 0, 1, 4);

        spacerItem12 = new QSpacerItem(16, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout3->addItem(spacerItem12, 1, 0, 1, 1);

        RandomizeNoteCheckBox = new QCheckBox(Randomize);
        RandomizeNoteCheckBox->setObjectName("RandomizeNoteCheckBox");

        gridLayout3->addWidget(RandomizeNoteCheckBox, 1, 1, 1, 1);

        RandomizeNoteSpinBox = new QDoubleSpinBox(Randomize);
        RandomizeNoteSpinBox->setObjectName("RandomizeNoteSpinBox");
        RandomizeNoteSpinBox->setMinimumSize(QSize(60, 0));
        RandomizeNoteSpinBox->setAccelerated(true);
        RandomizeNoteSpinBox->setDecimals(1);
        RandomizeNoteSpinBox->setMaximum(150.000000000000000);
        RandomizeNoteSpinBox->setSingleStep(0.100000000000000);

        gridLayout3->addWidget(RandomizeNoteSpinBox, 1, 2, 1, 1);

        spacerItem13 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout3->addItem(spacerItem13, 1, 3, 4, 1);

        RandomizeTimeCheckBox = new QCheckBox(Randomize);
        RandomizeTimeCheckBox->setObjectName("RandomizeTimeCheckBox");

        gridLayout3->addWidget(RandomizeTimeCheckBox, 2, 1, 1, 1);

        RandomizeTimeSpinBox = new QDoubleSpinBox(Randomize);
        RandomizeTimeSpinBox->setObjectName("RandomizeTimeSpinBox");
        RandomizeTimeSpinBox->setMinimumSize(QSize(60, 0));
        RandomizeTimeSpinBox->setAccelerated(true);
        RandomizeTimeSpinBox->setDecimals(1);
        RandomizeTimeSpinBox->setMaximum(150.000000000000000);
        RandomizeTimeSpinBox->setSingleStep(0.100000000000000);

        gridLayout3->addWidget(RandomizeTimeSpinBox, 2, 2, 1, 1);

        RandomizeDurationCheckBox = new QCheckBox(Randomize);
        RandomizeDurationCheckBox->setObjectName("RandomizeDurationCheckBox");

        gridLayout3->addWidget(RandomizeDurationCheckBox, 3, 1, 1, 1);

        RandomizeDurationSpinBox = new QDoubleSpinBox(Randomize);
        RandomizeDurationSpinBox->setObjectName("RandomizeDurationSpinBox");
        RandomizeDurationSpinBox->setMinimumSize(QSize(60, 0));
        RandomizeDurationSpinBox->setAccelerated(true);
        RandomizeDurationSpinBox->setDecimals(1);
        RandomizeDurationSpinBox->setMaximum(150.000000000000000);
        RandomizeDurationSpinBox->setSingleStep(0.100000000000000);

        gridLayout3->addWidget(RandomizeDurationSpinBox, 3, 2, 1, 1);

        RandomizeValueCheckBox = new QCheckBox(Randomize);
        RandomizeValueCheckBox->setObjectName("RandomizeValueCheckBox");

        gridLayout3->addWidget(RandomizeValueCheckBox, 4, 1, 1, 1);

        RandomizeValueSpinBox = new QDoubleSpinBox(Randomize);
        RandomizeValueSpinBox->setObjectName("RandomizeValueSpinBox");
        RandomizeValueSpinBox->setMinimumSize(QSize(60, 0));
        RandomizeValueSpinBox->setAccelerated(true);
        RandomizeValueSpinBox->setDecimals(1);
        RandomizeValueSpinBox->setMaximum(150.000000000000000);
        RandomizeValueSpinBox->setSingleStep(0.100000000000000);

        gridLayout3->addWidget(RandomizeValueSpinBox, 4, 2, 1, 1);

        spacerItem14 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout3->addItem(spacerItem14, 5, 1, 1, 3);

        ToolTabWidget->addTab(Randomize, QString());
        Resize = new QWidget();
        Resize->setObjectName("Resize");
        gridLayout4 = new QGridLayout(Resize);
        gridLayout4->setSpacing(4);
        gridLayout4->setContentsMargins(4, 4, 4, 4);
        gridLayout4->setObjectName("gridLayout4");
        ResizeCheckBox = new QCheckBox(Resize);
        ResizeCheckBox->setObjectName("ResizeCheckBox");
        ResizeCheckBox->setFont(font);

        gridLayout4->addWidget(ResizeCheckBox, 0, 0, 1, 7);

        spacerItem15 = new QSpacerItem(16, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout4->addItem(spacerItem15, 1, 0, 1, 1);

        ResizeDurationCheckBox = new QCheckBox(Resize);
        ResizeDurationCheckBox->setObjectName("ResizeDurationCheckBox");

        gridLayout4->addWidget(ResizeDurationCheckBox, 1, 1, 1, 1);

        ResizeDurationSpinBox = new qtractorTimeSpinBox(Resize);
        ResizeDurationSpinBox->setObjectName("ResizeDurationSpinBox");
        ResizeDurationSpinBox->setMinimumSize(QSize(120, 0));

        gridLayout4->addWidget(ResizeDurationSpinBox, 1, 2, 1, 2);

        ResizeDurationFormatComboBox = new QComboBox(Resize);
        ResizeDurationFormatComboBox->addItem(QString());
        ResizeDurationFormatComboBox->addItem(QString());
        ResizeDurationFormatComboBox->addItem(QString());
        ResizeDurationFormatComboBox->setObjectName("ResizeDurationFormatComboBox");

        gridLayout4->addWidget(ResizeDurationFormatComboBox, 1, 4, 1, 2);

        spacerItem16 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout4->addItem(spacerItem16, 1, 6, 1, 2);

        ResizeValueCheckBox = new QCheckBox(Resize);
        ResizeValueCheckBox->setObjectName("ResizeValueCheckBox");

        gridLayout4->addWidget(ResizeValueCheckBox, 2, 1, 1, 1);

        ResizeValueSpinBox = new QSpinBox(Resize);
        ResizeValueSpinBox->setObjectName("ResizeValueSpinBox");
        ResizeValueSpinBox->setMinimumSize(QSize(60, 0));
        ResizeValueSpinBox->setMaximum(127);

        gridLayout4->addWidget(ResizeValueSpinBox, 2, 2, 1, 1);

        ResizeValue2ComboBox = new QComboBox(Resize);
        ResizeValue2ComboBox->addItem(QString());
        ResizeValue2ComboBox->addItem(QString());
        ResizeValue2ComboBox->setObjectName("ResizeValue2ComboBox");

        gridLayout4->addWidget(ResizeValue2ComboBox, 2, 3, 1, 2);

        ResizeValue2SpinBox = new QSpinBox(Resize);
        ResizeValue2SpinBox->setObjectName("ResizeValue2SpinBox");
        ResizeValue2SpinBox->setMinimumSize(QSize(60, 0));
        ResizeValue2SpinBox->setMaximum(127);

        gridLayout4->addWidget(ResizeValue2SpinBox, 2, 5, 1, 2);

        spacerItem17 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout4->addItem(spacerItem17, 2, 7, 1, 1);

        ResizeLegatoCheckBox = new QCheckBox(Resize);
        ResizeLegatoCheckBox->setObjectName("ResizeLegatoCheckBox");

        gridLayout4->addWidget(ResizeLegatoCheckBox, 3, 1, 1, 1);

        ResizeLegatoTypeComboBox = new QComboBox(Resize);
        ResizeLegatoTypeComboBox->addItem(QString());
        ResizeLegatoTypeComboBox->addItem(QString());
        ResizeLegatoTypeComboBox->addItem(QString());
        ResizeLegatoTypeComboBox->setObjectName("ResizeLegatoTypeComboBox");
        ResizeLegatoTypeComboBox->setMinimumSize(QSize(60, 0));

        gridLayout4->addWidget(ResizeLegatoTypeComboBox, 3, 2, 1, 1);

        ResizeLegatoLengthComboBox = new QComboBox(Resize);
        ResizeLegatoLengthComboBox->setObjectName("ResizeLegatoLengthComboBox");

        gridLayout4->addWidget(ResizeLegatoLengthComboBox, 3, 3, 1, 2);

        ResizeLegatoModeComboBox = new QComboBox(Resize);
        ResizeLegatoModeComboBox->addItem(QString());
        ResizeLegatoModeComboBox->addItem(QString());
        ResizeLegatoModeComboBox->setObjectName("ResizeLegatoModeComboBox");

        gridLayout4->addWidget(ResizeLegatoModeComboBox, 3, 5, 1, 2);

        spacerItem18 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout4->addItem(spacerItem18, 3, 7, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(4);
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName("hboxLayout1");
        ResizeJoinCheckBox = new QCheckBox(Resize);
        ResizeJoinCheckBox->setObjectName("ResizeJoinCheckBox");

        hboxLayout1->addWidget(ResizeJoinCheckBox);

        ResizeSplitCheckBox = new QCheckBox(Resize);
        ResizeSplitCheckBox->setObjectName("ResizeSplitCheckBox");

        hboxLayout1->addWidget(ResizeSplitCheckBox);

        ResizeSplitLengthComboBox = new QComboBox(Resize);
        ResizeSplitLengthComboBox->setObjectName("ResizeSplitLengthComboBox");

        hboxLayout1->addWidget(ResizeSplitLengthComboBox);

        ResizeSplitOffsetComboBox = new QComboBox(Resize);
        ResizeSplitOffsetComboBox->addItem(QString());
        ResizeSplitOffsetComboBox->addItem(QString());
        ResizeSplitOffsetComboBox->setObjectName("ResizeSplitOffsetComboBox");

        hboxLayout1->addWidget(ResizeSplitOffsetComboBox);

        spacerItem19 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem19);


        gridLayout4->addLayout(hboxLayout1, 4, 1, 1, 6);

        spacerItem20 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout4->addItem(spacerItem20, 5, 1, 1, 6);

        ToolTabWidget->addTab(Resize, QString());
        Rescale = new QWidget();
        Rescale->setObjectName("Rescale");
        gridLayout5 = new QGridLayout(Rescale);
        gridLayout5->setSpacing(4);
        gridLayout5->setContentsMargins(4, 4, 4, 4);
        gridLayout5->setObjectName("gridLayout5");
        RescaleCheckBox = new QCheckBox(Rescale);
        RescaleCheckBox->setObjectName("RescaleCheckBox");
        RescaleCheckBox->setFont(font);

        gridLayout5->addWidget(RescaleCheckBox, 0, 0, 1, 4);

        spacerItem21 = new QSpacerItem(16, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout5->addItem(spacerItem21, 1, 0, 1, 1);

        RescaleTimeCheckBox = new QCheckBox(Rescale);
        RescaleTimeCheckBox->setObjectName("RescaleTimeCheckBox");

        gridLayout5->addWidget(RescaleTimeCheckBox, 1, 1, 1, 1);

        RescaleTimeSpinBox = new QDoubleSpinBox(Rescale);
        RescaleTimeSpinBox->setObjectName("RescaleTimeSpinBox");
        RescaleTimeSpinBox->setMinimumSize(QSize(60, 0));
        RescaleTimeSpinBox->setAccelerated(true);
        RescaleTimeSpinBox->setDecimals(1);
        RescaleTimeSpinBox->setMinimum(0.100000000000000);
        RescaleTimeSpinBox->setMaximum(1000.000000000000000);
        RescaleTimeSpinBox->setSingleStep(0.100000000000000);
        RescaleTimeSpinBox->setValue(100.000000000000000);

        gridLayout5->addWidget(RescaleTimeSpinBox, 1, 2, 1, 1);

        spacerItem22 = new QSpacerItem(240, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout5->addItem(spacerItem22, 1, 3, 3, 1);

        RescaleDurationCheckBox = new QCheckBox(Rescale);
        RescaleDurationCheckBox->setObjectName("RescaleDurationCheckBox");

        gridLayout5->addWidget(RescaleDurationCheckBox, 2, 1, 1, 1);

        RescaleDurationSpinBox = new QDoubleSpinBox(Rescale);
        RescaleDurationSpinBox->setObjectName("RescaleDurationSpinBox");
        RescaleDurationSpinBox->setMinimumSize(QSize(60, 0));
        RescaleDurationSpinBox->setAccelerated(true);
        RescaleDurationSpinBox->setDecimals(1);
        RescaleDurationSpinBox->setMinimum(0.100000000000000);
        RescaleDurationSpinBox->setMaximum(1000.000000000000000);
        RescaleDurationSpinBox->setSingleStep(0.100000000000000);
        RescaleDurationSpinBox->setValue(100.000000000000000);

        gridLayout5->addWidget(RescaleDurationSpinBox, 2, 2, 1, 1);

        RescaleValueCheckBox = new QCheckBox(Rescale);
        RescaleValueCheckBox->setObjectName("RescaleValueCheckBox");

        gridLayout5->addWidget(RescaleValueCheckBox, 3, 1, 1, 1);

        RescaleValueSpinBox = new QDoubleSpinBox(Rescale);
        RescaleValueSpinBox->setObjectName("RescaleValueSpinBox");
        RescaleValueSpinBox->setMinimumSize(QSize(60, 0));
        RescaleValueSpinBox->setAccelerated(true);
        RescaleValueSpinBox->setDecimals(1);
        RescaleValueSpinBox->setMinimum(0.100000000000000);
        RescaleValueSpinBox->setMaximum(1000.000000000000000);
        RescaleValueSpinBox->setSingleStep(0.100000000000000);
        RescaleValueSpinBox->setValue(100.000000000000000);

        gridLayout5->addWidget(RescaleValueSpinBox, 3, 2, 1, 1);

        RescaleInvertCheckBox = new QCheckBox(Rescale);
        RescaleInvertCheckBox->setObjectName("RescaleInvertCheckBox");

        gridLayout5->addWidget(RescaleInvertCheckBox, 4, 2, 1, 2);

        spacerItem23 = new QSpacerItem(360, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout5->addItem(spacerItem23, 5, 1, 1, 3);

        ToolTabWidget->addTab(Rescale, QString());
        Timeshift = new QWidget();
        Timeshift->setObjectName("Timeshift");
        gridLayout6 = new QGridLayout(Timeshift);
        gridLayout6->setSpacing(4);
        gridLayout6->setContentsMargins(4, 4, 4, 4);
        gridLayout6->setObjectName("gridLayout6");
        TimeshiftCheckBox = new QCheckBox(Timeshift);
        TimeshiftCheckBox->setObjectName("TimeshiftCheckBox");
        TimeshiftCheckBox->setFont(font);

        gridLayout6->addWidget(TimeshiftCheckBox, 0, 0, 1, 5);

        spacerItem24 = new QSpacerItem(16, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout6->addItem(spacerItem24, 1, 0, 1, 1);

        TimeshiftLabel = new QLabel(Timeshift);
        TimeshiftLabel->setObjectName("TimeshiftLabel");

        gridLayout6->addWidget(TimeshiftLabel, 1, 1, 1, 1);

        TimeshiftSpinBox = new QDoubleSpinBox(Timeshift);
        TimeshiftSpinBox->setObjectName("TimeshiftSpinBox");
        TimeshiftSpinBox->setMinimumSize(QSize(60, 0));
        TimeshiftSpinBox->setAccelerated(true);
        TimeshiftSpinBox->setDecimals(1);
        TimeshiftSpinBox->setMinimum(-100.000000000000000);
        TimeshiftSpinBox->setMaximum(100.000000000000000);
        TimeshiftSpinBox->setSingleStep(0.100000000000000);
        TimeshiftSpinBox->setValue(0.000000000000000);

        gridLayout6->addWidget(TimeshiftSpinBox, 1, 2, 1, 1);

        TimeshiftSlider = new QSlider(Timeshift);
        TimeshiftSlider->setObjectName("TimeshiftSlider");
        TimeshiftSlider->setMinimum(-10000);
        TimeshiftSlider->setMaximum(10000);
        TimeshiftSlider->setSingleStep(10);
        TimeshiftSlider->setPageStep(100);
        TimeshiftSlider->setValue(0);
        TimeshiftSlider->setOrientation(Qt::Horizontal);

        gridLayout6->addWidget(TimeshiftSlider, 1, 3, 1, 1);

        TimeshiftFrame = new QFrame(Timeshift);
        TimeshiftFrame->setObjectName("TimeshiftFrame");
        TimeshiftFrame->setMinimumSize(QSize(80, 80));
        TimeshiftFrame->setMaximumSize(QSize(80, 80));
        TimeshiftFrame->setFrameShape(QFrame::Panel);
        TimeshiftFrame->setFrameShadow(QFrame::Sunken);

        gridLayout6->addWidget(TimeshiftFrame, 1, 4, 2, 1);

        TimeshiftText = new QLabel(Timeshift);
        TimeshiftText->setObjectName("TimeshiftText");

        gridLayout6->addWidget(TimeshiftText, 2, 2, 1, 2);

        TimeshiftDurationCheckBox = new QCheckBox(Timeshift);
        TimeshiftDurationCheckBox->setObjectName("TimeshiftDurationCheckBox");

        gridLayout6->addWidget(TimeshiftDurationCheckBox, 3, 1, 1, 4);

        spacerItem25 = new QSpacerItem(360, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout6->addItem(spacerItem25, 4, 1, 1, 4);

        ToolTabWidget->addTab(Timeshift, QString());
        Temporamp = new QWidget();
        Temporamp->setObjectName("Temporamp");
        gridLayout7 = new QGridLayout(Temporamp);
        gridLayout7->setSpacing(4);
        gridLayout7->setContentsMargins(4, 4, 4, 4);
        gridLayout7->setObjectName("gridLayout7");
        TemporampCheckBox = new QCheckBox(Temporamp);
        TemporampCheckBox->setObjectName("TemporampCheckBox");
        TemporampCheckBox->setFont(font);

        gridLayout7->addWidget(TemporampCheckBox, 0, 0, 1, 7);

        spacerItem26 = new QSpacerItem(16, 20, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        gridLayout7->addItem(spacerItem26, 1, 0, 1, 1);

        TemporampFromLabel = new QLabel(Temporamp);
        TemporampFromLabel->setObjectName("TemporampFromLabel");

        gridLayout7->addWidget(TemporampFromLabel, 1, 1, 1, 1);

        TemporampFromSpinBox = new qtractorTempoSpinBox(Temporamp);
        TemporampFromSpinBox->setObjectName("TemporampFromSpinBox");
        TemporampFromSpinBox->setMinimumSize(QSize(80, 0));

        gridLayout7->addWidget(TemporampFromSpinBox, 1, 2, 1, 1);

        TemporampToLabel = new QLabel(Temporamp);
        TemporampToLabel->setObjectName("TemporampToLabel");

        gridLayout7->addWidget(TemporampToLabel, 1, 3, 1, 1);

        TemporampToSpinBox = new qtractorTempoSpinBox(Temporamp);
        TemporampToSpinBox->setObjectName("TemporampToSpinBox");
        TemporampToSpinBox->setMinimumSize(QSize(80, 0));

        gridLayout7->addWidget(TemporampToSpinBox, 1, 4, 1, 1);

        spacerItem27 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout7->addItem(spacerItem27, 1, 6, 1, 1);

        TemporampText = new QLabel(Temporamp);
        TemporampText->setObjectName("TemporampText");

        gridLayout7->addWidget(TemporampText, 2, 2, 1, 5);

        TemporampDurationCheckBox = new QCheckBox(Temporamp);
        TemporampDurationCheckBox->setObjectName("TemporampDurationCheckBox");

        gridLayout7->addWidget(TemporampDurationCheckBox, 3, 1, 1, 5);

        spacerItem28 = new QSpacerItem(360, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout7->addItem(spacerItem28, 4, 1, 1, 6);

        ToolTabWidget->addTab(Temporamp, QString());

        vboxLayout->addWidget(ToolTabWidget);

        DialogButtonBox = new QDialogButtonBox(qtractorMidiToolsForm);
        DialogButtonBox->setObjectName("DialogButtonBox");
        DialogButtonBox->setOrientation(Qt::Horizontal);
        DialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(DialogButtonBox);

        QWidget::setTabOrder(PresetNameComboBox, PresetSaveToolButton);
        QWidget::setTabOrder(PresetSaveToolButton, PresetDeleteToolButton);
        QWidget::setTabOrder(PresetDeleteToolButton, ToolTabWidget);
        QWidget::setTabOrder(ToolTabWidget, QuantizeCheckBox);
        QWidget::setTabOrder(QuantizeCheckBox, QuantizeTimeCheckBox);
        QWidget::setTabOrder(QuantizeTimeCheckBox, QuantizeTimeComboBox);
        QWidget::setTabOrder(QuantizeTimeComboBox, QuantizeTimeSpinBox);
        QWidget::setTabOrder(QuantizeTimeSpinBox, QuantizeDurationCheckBox);
        QWidget::setTabOrder(QuantizeDurationCheckBox, QuantizeDurationComboBox);
        QWidget::setTabOrder(QuantizeDurationComboBox, QuantizeDurationSpinBox);
        QWidget::setTabOrder(QuantizeDurationSpinBox, QuantizeSwingCheckBox);
        QWidget::setTabOrder(QuantizeSwingCheckBox, QuantizeSwingComboBox);
        QWidget::setTabOrder(QuantizeSwingComboBox, QuantizeSwingSpinBox);
        QWidget::setTabOrder(QuantizeSwingSpinBox, QuantizeSwingTypeComboBox);
        QWidget::setTabOrder(QuantizeSwingTypeComboBox, QuantizeScaleCheckBox);
        QWidget::setTabOrder(QuantizeScaleCheckBox, QuantizeScaleKeyComboBox);
        QWidget::setTabOrder(QuantizeScaleKeyComboBox, QuantizeScaleComboBox);
        QWidget::setTabOrder(QuantizeScaleComboBox, TransposeCheckBox);
        QWidget::setTabOrder(TransposeCheckBox, TransposeNoteCheckBox);
        QWidget::setTabOrder(TransposeNoteCheckBox, TransposeNoteSpinBox);
        QWidget::setTabOrder(TransposeNoteSpinBox, TransposeTimeCheckBox);
        QWidget::setTabOrder(TransposeTimeCheckBox, TransposeTimeSpinBox);
        QWidget::setTabOrder(TransposeTimeSpinBox, TransposeFormatComboBox);
        QWidget::setTabOrder(TransposeFormatComboBox, TransposeReverseCheckBox);
        QWidget::setTabOrder(TransposeReverseCheckBox, NormalizeCheckBox);
        QWidget::setTabOrder(NormalizeCheckBox, NormalizePercentCheckBox);
        QWidget::setTabOrder(NormalizePercentCheckBox, NormalizePercentSpinBox);
        QWidget::setTabOrder(NormalizePercentSpinBox, NormalizeValueCheckBox);
        QWidget::setTabOrder(NormalizeValueCheckBox, NormalizeValueSpinBox);
        QWidget::setTabOrder(NormalizeValueSpinBox, NormalizeCompressCheckBox);
        QWidget::setTabOrder(NormalizeCompressCheckBox, RandomizeCheckBox);
        QWidget::setTabOrder(RandomizeCheckBox, RandomizeNoteCheckBox);
        QWidget::setTabOrder(RandomizeNoteCheckBox, RandomizeNoteSpinBox);
        QWidget::setTabOrder(RandomizeNoteSpinBox, RandomizeTimeCheckBox);
        QWidget::setTabOrder(RandomizeTimeCheckBox, RandomizeTimeSpinBox);
        QWidget::setTabOrder(RandomizeTimeSpinBox, RandomizeDurationCheckBox);
        QWidget::setTabOrder(RandomizeDurationCheckBox, RandomizeDurationSpinBox);
        QWidget::setTabOrder(RandomizeDurationSpinBox, RandomizeValueCheckBox);
        QWidget::setTabOrder(RandomizeValueCheckBox, RandomizeValueSpinBox);
        QWidget::setTabOrder(RandomizeValueSpinBox, ResizeCheckBox);
        QWidget::setTabOrder(ResizeCheckBox, ResizeDurationCheckBox);
        QWidget::setTabOrder(ResizeDurationCheckBox, ResizeDurationSpinBox);
        QWidget::setTabOrder(ResizeDurationSpinBox, ResizeDurationFormatComboBox);
        QWidget::setTabOrder(ResizeDurationFormatComboBox, ResizeValueCheckBox);
        QWidget::setTabOrder(ResizeValueCheckBox, ResizeValueSpinBox);
        QWidget::setTabOrder(ResizeValueSpinBox, ResizeValue2ComboBox);
        QWidget::setTabOrder(ResizeValue2ComboBox, ResizeValue2SpinBox);
        QWidget::setTabOrder(ResizeValue2SpinBox, ResizeLegatoCheckBox);
        QWidget::setTabOrder(ResizeLegatoCheckBox, ResizeLegatoTypeComboBox);
        QWidget::setTabOrder(ResizeLegatoTypeComboBox, ResizeLegatoLengthComboBox);
        QWidget::setTabOrder(ResizeLegatoLengthComboBox, ResizeLegatoModeComboBox);
        QWidget::setTabOrder(ResizeLegatoModeComboBox, ResizeJoinCheckBox);
        QWidget::setTabOrder(ResizeJoinCheckBox, ResizeSplitCheckBox);
        QWidget::setTabOrder(ResizeSplitCheckBox, ResizeSplitLengthComboBox);
        QWidget::setTabOrder(ResizeSplitLengthComboBox, ResizeSplitOffsetComboBox);
        QWidget::setTabOrder(ResizeSplitOffsetComboBox, RescaleCheckBox);
        QWidget::setTabOrder(RescaleCheckBox, RescaleTimeCheckBox);
        QWidget::setTabOrder(RescaleTimeCheckBox, RescaleTimeSpinBox);
        QWidget::setTabOrder(RescaleTimeSpinBox, RescaleDurationCheckBox);
        QWidget::setTabOrder(RescaleDurationCheckBox, RescaleDurationSpinBox);
        QWidget::setTabOrder(RescaleDurationSpinBox, RescaleValueCheckBox);
        QWidget::setTabOrder(RescaleValueCheckBox, RescaleValueSpinBox);
        QWidget::setTabOrder(RescaleValueSpinBox, RescaleInvertCheckBox);
        QWidget::setTabOrder(RescaleInvertCheckBox, TimeshiftCheckBox);
        QWidget::setTabOrder(TimeshiftCheckBox, TimeshiftSpinBox);
        QWidget::setTabOrder(TimeshiftSpinBox, TimeshiftSlider);
        QWidget::setTabOrder(TimeshiftSlider, TimeshiftDurationCheckBox);
        QWidget::setTabOrder(TimeshiftDurationCheckBox, TemporampFromSpinBox);
        QWidget::setTabOrder(TemporampFromSpinBox, TemporampToSpinBox);
        QWidget::setTabOrder(TemporampToSpinBox, TemporampDurationCheckBox);
        QWidget::setTabOrder(TemporampDurationCheckBox, DialogButtonBox);

        retranslateUi(qtractorMidiToolsForm);

        ToolTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(qtractorMidiToolsForm);
    } // setupUi

    void retranslateUi(QDialog *qtractorMidiToolsForm)
    {
        qtractorMidiToolsForm->setWindowTitle(QCoreApplication::translate("qtractorMidiToolsForm", "MIDI Tools", nullptr));
#if QT_CONFIG(tooltip)
        PresetNameComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Preset name", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        PresetSaveToolButton->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Save preset", nullptr));
#endif // QT_CONFIG(tooltip)
        PresetSaveToolButton->setText(QString());
#if QT_CONFIG(tooltip)
        PresetDeleteToolButton->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Delete preset", nullptr));
#endif // QT_CONFIG(tooltip)
        PresetDeleteToolButton->setText(QString());
#if QT_CONFIG(tooltip)
        QuantizeCheckBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Quantize selected events", nullptr));
#endif // QT_CONFIG(tooltip)
        QuantizeCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Quantize", nullptr));
        QuantizeTimeCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Time:", nullptr));
#if QT_CONFIG(tooltip)
        QuantizeTimeComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Quantize time", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        QuantizeTimeSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Quantize time percent", nullptr));
#endif // QT_CONFIG(tooltip)
        QuantizeTimeSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        QuantizeDurationCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Duration:", nullptr));
#if QT_CONFIG(tooltip)
        QuantizeDurationComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Quantize duration", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        QuantizeDurationSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Quantize duration percent", nullptr));
#endif // QT_CONFIG(tooltip)
        QuantizeDurationSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        QuantizeSwingCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "S&wing:", nullptr));
#if QT_CONFIG(tooltip)
        QuantizeSwingComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Swing-quantize time", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        QuantizeSwingSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Swing-quantize percent", nullptr));
#endif // QT_CONFIG(tooltip)
        QuantizeSwingSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        QuantizeSwingTypeComboBox->setItemText(0, QCoreApplication::translate("qtractorMidiToolsForm", "Linear", nullptr));
        QuantizeSwingTypeComboBox->setItemText(1, QCoreApplication::translate("qtractorMidiToolsForm", "Quadratic", nullptr));
        QuantizeSwingTypeComboBox->setItemText(2, QCoreApplication::translate("qtractorMidiToolsForm", "Cubic", nullptr));

#if QT_CONFIG(tooltip)
        QuantizeSwingTypeComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Swing-quantize type", nullptr));
#endif // QT_CONFIG(tooltip)
        QuantizeScaleCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Scale:", nullptr));
#if QT_CONFIG(tooltip)
        QuantizeScaleKeyComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Scale-quantize key", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        QuantizeScaleComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Scale-quantize type", nullptr));
#endif // QT_CONFIG(tooltip)
        ToolTabWidget->setTabText(ToolTabWidget->indexOf(Quantize), QCoreApplication::translate("qtractorMidiToolsForm", "&Quantize", nullptr));
#if QT_CONFIG(tooltip)
        TransposeCheckBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Transpose selected events", nullptr));
#endif // QT_CONFIG(tooltip)
        TransposeCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Transpose", nullptr));
        TransposeNoteCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Note:", nullptr));
#if QT_CONFIG(tooltip)
        TransposeNoteSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Transpose note", nullptr));
#endif // QT_CONFIG(tooltip)
        TransposeTimeCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Time:", nullptr));
#if QT_CONFIG(tooltip)
        TransposeTimeSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Transpose time", nullptr));
#endif // QT_CONFIG(tooltip)
        TransposeFormatComboBox->setItemText(0, QCoreApplication::translate("qtractorMidiToolsForm", "Frames", nullptr));
        TransposeFormatComboBox->setItemText(1, QCoreApplication::translate("qtractorMidiToolsForm", "Time", nullptr));
        TransposeFormatComboBox->setItemText(2, QCoreApplication::translate("qtractorMidiToolsForm", "BBT", nullptr));

#if QT_CONFIG(tooltip)
        TransposeFormatComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Transpose time format", nullptr));
#endif // QT_CONFIG(tooltip)
        TransposeReverseCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Reverse", nullptr));
        ToolTabWidget->setTabText(ToolTabWidget->indexOf(Transpose), QCoreApplication::translate("qtractorMidiToolsForm", "&Transpose", nullptr));
#if QT_CONFIG(tooltip)
        NormalizeCheckBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Normalize selected events", nullptr));
#endif // QT_CONFIG(tooltip)
        NormalizeCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Normalize", nullptr));
        NormalizePercentCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Percent:", nullptr));
#if QT_CONFIG(tooltip)
        NormalizePercentSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Normalize percent", nullptr));
#endif // QT_CONFIG(tooltip)
        NormalizePercentSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        NormalizeValueCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Value:", nullptr));
#if QT_CONFIG(tooltip)
        NormalizeValueSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Normalize value", nullptr));
#endif // QT_CONFIG(tooltip)
        NormalizeCompressCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Compress", nullptr));
        ToolTabWidget->setTabText(ToolTabWidget->indexOf(Normalize), QCoreApplication::translate("qtractorMidiToolsForm", "&Normalize", nullptr));
#if QT_CONFIG(tooltip)
        RandomizeCheckBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Randomize selected events", nullptr));
#endif // QT_CONFIG(tooltip)
        RandomizeCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Randomize", nullptr));
        RandomizeNoteCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Note:", nullptr));
#if QT_CONFIG(tooltip)
        RandomizeNoteSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Randomize note/pitch", nullptr));
#endif // QT_CONFIG(tooltip)
        RandomizeNoteSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        RandomizeTimeCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Time:", nullptr));
#if QT_CONFIG(tooltip)
        RandomizeTimeSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Randomize time", nullptr));
#endif // QT_CONFIG(tooltip)
        RandomizeTimeSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        RandomizeDurationCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Duration:", nullptr));
#if QT_CONFIG(tooltip)
        RandomizeDurationSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Randomize duration", nullptr));
#endif // QT_CONFIG(tooltip)
        RandomizeDurationSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        RandomizeValueCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Value:", nullptr));
#if QT_CONFIG(tooltip)
        RandomizeValueSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Randomize value", nullptr));
#endif // QT_CONFIG(tooltip)
        RandomizeValueSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        ToolTabWidget->setTabText(ToolTabWidget->indexOf(Randomize), QCoreApplication::translate("qtractorMidiToolsForm", "&Randomize", nullptr));
#if QT_CONFIG(tooltip)
        ResizeCheckBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Resize selected events", nullptr));
#endif // QT_CONFIG(tooltip)
        ResizeCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "Resi&ze", nullptr));
        ResizeDurationCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Duration:", nullptr));
#if QT_CONFIG(tooltip)
        ResizeDurationSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Resize duration", nullptr));
#endif // QT_CONFIG(tooltip)
        ResizeDurationFormatComboBox->setItemText(0, QCoreApplication::translate("qtractorMidiToolsForm", "Frames", nullptr));
        ResizeDurationFormatComboBox->setItemText(1, QCoreApplication::translate("qtractorMidiToolsForm", "Time", nullptr));
        ResizeDurationFormatComboBox->setItemText(2, QCoreApplication::translate("qtractorMidiToolsForm", "BBT", nullptr));

#if QT_CONFIG(tooltip)
        ResizeDurationFormatComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Resize duration format", nullptr));
#endif // QT_CONFIG(tooltip)
        ResizeValueCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Value:", nullptr));
#if QT_CONFIG(tooltip)
        ResizeValueSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Resize value", nullptr));
#endif // QT_CONFIG(tooltip)
        ResizeValue2ComboBox->setItemText(0, QCoreApplication::translate("qtractorMidiToolsForm", "Flat", nullptr));
        ResizeValue2ComboBox->setItemText(1, QCoreApplication::translate("qtractorMidiToolsForm", "Ramp", nullptr));

#if QT_CONFIG(tooltip)
        ResizeValue2ComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Resize value mode", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        ResizeValue2SpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Resize final value", nullptr));
#endif // QT_CONFIG(tooltip)
        ResizeLegatoCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Legato:", nullptr));
        ResizeLegatoTypeComboBox->setItemText(0, QCoreApplication::translate("qtractorMidiToolsForm", "Normal", nullptr));
        ResizeLegatoTypeComboBox->setItemText(1, QCoreApplication::translate("qtractorMidiToolsForm", "Trim", nullptr));
        ResizeLegatoTypeComboBox->setItemText(2, QCoreApplication::translate("qtractorMidiToolsForm", "Extend", nullptr));

#if QT_CONFIG(tooltip)
        ResizeLegatoTypeComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Legato trim/extend type", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        ResizeLegatoLengthComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Legato trim/extend length", nullptr));
#endif // QT_CONFIG(tooltip)
        ResizeLegatoModeComboBox->setItemText(0, QCoreApplication::translate("qtractorMidiToolsForm", "Mono", nullptr));
        ResizeLegatoModeComboBox->setItemText(1, QCoreApplication::translate("qtractorMidiToolsForm", "Poly", nullptr));

#if QT_CONFIG(tooltip)
        ResizeLegatoModeComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Legato mode", nullptr));
#endif // QT_CONFIG(tooltip)
        ResizeJoinCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Join", nullptr));
        ResizeSplitCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Split:", nullptr));
#if QT_CONFIG(tooltip)
        ResizeSplitLengthComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Split notes length", nullptr));
#endif // QT_CONFIG(tooltip)
        ResizeSplitOffsetComboBox->setItemText(0, QCoreApplication::translate("qtractorMidiToolsForm", "Relative", nullptr));
        ResizeSplitOffsetComboBox->setItemText(1, QCoreApplication::translate("qtractorMidiToolsForm", "Absolute", nullptr));

#if QT_CONFIG(tooltip)
        ResizeSplitOffsetComboBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Split notes offset", nullptr));
#endif // QT_CONFIG(tooltip)
        ToolTabWidget->setTabText(ToolTabWidget->indexOf(Resize), QCoreApplication::translate("qtractorMidiToolsForm", "Resi&ze", nullptr));
#if QT_CONFIG(tooltip)
        RescaleCheckBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Rescale selected events", nullptr));
#endif // QT_CONFIG(tooltip)
        RescaleCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "Re&scale", nullptr));
        RescaleTimeCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Time:", nullptr));
#if QT_CONFIG(tooltip)
        RescaleTimeSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Rescale time", nullptr));
#endif // QT_CONFIG(tooltip)
        RescaleTimeSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        RescaleDurationCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Duration:", nullptr));
#if QT_CONFIG(tooltip)
        RescaleDurationSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Rescale duration", nullptr));
#endif // QT_CONFIG(tooltip)
        RescaleDurationSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        RescaleValueCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Value:", nullptr));
#if QT_CONFIG(tooltip)
        RescaleValueSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Rescale value", nullptr));
#endif // QT_CONFIG(tooltip)
        RescaleValueSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
        RescaleInvertCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "&Invert", nullptr));
        ToolTabWidget->setTabText(ToolTabWidget->indexOf(Rescale), QCoreApplication::translate("qtractorMidiToolsForm", "Re&scale", nullptr));
#if QT_CONFIG(tooltip)
        TimeshiftCheckBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Timeshift selected events", nullptr));
#endif // QT_CONFIG(tooltip)
        TimeshiftCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "Timeshift", nullptr));
        TimeshiftLabel->setText(QCoreApplication::translate("qtractorMidiToolsForm", "P:", nullptr));
#if QT_CONFIG(tooltip)
        TimeshiftSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Timeshift parameter", nullptr));
#endif // QT_CONFIG(tooltip)
        TimeshiftSpinBox->setSuffix(QCoreApplication::translate("qtractorMidiToolsForm", " %", nullptr));
#if QT_CONFIG(tooltip)
        TimeshiftSlider->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Timeshift parameter (log)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        TimeshiftFrame->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Timeshift curve", nullptr));
#endif // QT_CONFIG(tooltip)
        TimeshiftText->setText(QCoreApplication::translate("qtractorMidiToolsForm", "P = 0 : no change.\n"
"P > 0 : accelerating shift.\n"
"P < 0 : slowing down shift.\n"
"Edit head/tail (blue) markers define the shift range.", nullptr));
        TimeshiftDurationCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "Timeshift duration", nullptr));
        ToolTabWidget->setTabText(ToolTabWidget->indexOf(Timeshift), QCoreApplication::translate("qtractorMidiToolsForm", "T&imeshift", nullptr));
#if QT_CONFIG(tooltip)
        TemporampCheckBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Tempo ramp selected events", nullptr));
#endif // QT_CONFIG(tooltip)
        TemporampCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "Tempo ramp", nullptr));
        TemporampFromLabel->setText(QCoreApplication::translate("qtractorMidiToolsForm", "From", nullptr));
#if QT_CONFIG(tooltip)
        TemporampFromSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Tempo ramp start", nullptr));
#endif // QT_CONFIG(tooltip)
        TemporampToLabel->setText(QCoreApplication::translate("qtractorMidiToolsForm", "to", nullptr));
#if QT_CONFIG(tooltip)
        TemporampToSpinBox->setToolTip(QCoreApplication::translate("qtractorMidiToolsForm", "Temporamp end", nullptr));
#endif // QT_CONFIG(tooltip)
        TemporampText->setText(QCoreApplication::translate("qtractorMidiToolsForm", "Edit head/tail (blue) markers define the ramp range.", nullptr));
        TemporampDurationCheckBox->setText(QCoreApplication::translate("qtractorMidiToolsForm", "Tempo ramp duration", nullptr));
        ToolTabWidget->setTabText(ToolTabWidget->indexOf(Temporamp), QCoreApplication::translate("qtractorMidiToolsForm", "T&empo ramp", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qtractorMidiToolsForm: public Ui_qtractorMidiToolsForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QTRACTORMIDITOOLSFORM_H
