/****************************************************************************
** Meta object code from reading C++ file 'qtractorMidiEditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorMidiEditor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorMidiEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qtractorMidiEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorMidiEditor::qt_create_metaobjectdata<qt_meta_tag_ZN18qtractorMidiEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorMidiEditor",
        "selectNotifySignal",
        "",
        "qtractorMidiEditor*",
        "changeNotifySignal",
        "sendNoteSignal",
        "sendNote",
        "iNote",
        "iVelocity",
        "bForce",
        "updateInstrumentNames",
        "zoomIn",
        "zoomOut",
        "zoomReset",
        "horizontalZoomInSlot",
        "horizontalZoomOutSlot",
        "horizontalZoomResetSlot",
        "verticalZoomInSlot",
        "verticalZoomOutSlot",
        "verticalZoomResetSlot",
        "horizontalSplitterSlot",
        "verticalSplitterSlot",
        "updateNotifySlot",
        "flags"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectNotifySignal'
        QtMocHelpers::SignalData<void(qtractorMidiEditor *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'changeNotifySignal'
        QtMocHelpers::SignalData<void(qtractorMidiEditor *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'sendNoteSignal'
        QtMocHelpers::SignalData<void(int, int, bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Int, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Slot 'sendNote'
        QtMocHelpers::SlotData<void(int, int, bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 }, { QMetaType::Bool, 9 },
        }}),
        // Slot 'sendNote'
        QtMocHelpers::SlotData<void(int, int)>(6, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'updateInstrumentNames'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomIn'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomOut'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'zoomReset'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'horizontalZoomInSlot'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'horizontalZoomOutSlot'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'horizontalZoomResetSlot'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'verticalZoomInSlot'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'verticalZoomOutSlot'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'verticalZoomResetSlot'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'horizontalSplitterSlot'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'verticalSplitterSlot'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateNotifySlot'
        QtMocHelpers::SlotData<void(unsigned int)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::UInt, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorMidiEditor, qt_meta_tag_ZN18qtractorMidiEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorMidiEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitter::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qtractorMidiEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qtractorMidiEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qtractorMidiEditorE_t>.metaTypes,
    nullptr
} };

void qtractorMidiEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorMidiEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectNotifySignal((*reinterpret_cast<std::add_pointer_t<qtractorMidiEditor*>>(_a[1]))); break;
        case 1: _t->changeNotifySignal((*reinterpret_cast<std::add_pointer_t<qtractorMidiEditor*>>(_a[1]))); break;
        case 2: _t->sendNoteSignal((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 3: _t->sendNote((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 4: _t->sendNote((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->updateInstrumentNames(); break;
        case 6: _t->zoomIn(); break;
        case 7: _t->zoomOut(); break;
        case 8: _t->zoomReset(); break;
        case 9: _t->horizontalZoomInSlot(); break;
        case 10: _t->horizontalZoomOutSlot(); break;
        case 11: _t->horizontalZoomResetSlot(); break;
        case 12: _t->verticalZoomInSlot(); break;
        case 13: _t->verticalZoomOutSlot(); break;
        case 14: _t->verticalZoomResetSlot(); break;
        case 15: _t->horizontalSplitterSlot(); break;
        case 16: _t->verticalSplitterSlot(); break;
        case 17: _t->updateNotifySlot((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< qtractorMidiEditor* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< qtractorMidiEditor* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qtractorMidiEditor::*)(qtractorMidiEditor * )>(_a, &qtractorMidiEditor::selectNotifySignal, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (qtractorMidiEditor::*)(qtractorMidiEditor * )>(_a, &qtractorMidiEditor::changeNotifySignal, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (qtractorMidiEditor::*)(int , int , bool )>(_a, &qtractorMidiEditor::sendNoteSignal, 2))
            return;
    }
}

const QMetaObject *qtractorMidiEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorMidiEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qtractorMidiEditorE_t>.strings))
        return static_cast<void*>(this);
    return QSplitter::qt_metacast(_clname);
}

int qtractorMidiEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void qtractorMidiEditor::selectNotifySignal(qtractorMidiEditor * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void qtractorMidiEditor::changeNotifySignal(qtractorMidiEditor * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void qtractorMidiEditor::sendNoteSignal(int _t1, int _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
