/****************************************************************************
** Meta object code from reading C++ file 'qtractorPluginListView.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorPluginListView.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorPluginListView.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22qtractorPluginListViewE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorPluginListView::qt_create_metaobjectdata<qt_meta_tag_ZN22qtractorPluginListViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorPluginListView",
        "contentsChanged",
        "",
        "addPlugin",
        "removePlugin",
        "moveUpPlugin",
        "moveDownPlugin",
        "activatePlugin",
        "activateAllPlugins",
        "deactivateAllPlugins",
        "removeAllPlugins",
        "loadPresetPlugin",
        "directAccessPlugin",
        "propertiesPlugin",
        "editPlugin",
        "importPlugins",
        "exportPlugins",
        "addAudioInsertPlugin",
        "addAudioAuxSendPlugin",
        "addMidiInsertPlugin",
        "addMidiAuxSendPlugin",
        "addMidiControlPlugin",
        "insertPluginOutputs",
        "insertPluginInputs",
        "midiControlAutoConnect",
        "audioOutputs",
        "audioOutputBus",
        "audioOutputBusName",
        "audioOutputAutoConnect",
        "dropMove",
        "dropCopy",
        "dropCancel",
        "itemDoubleClickedSlot",
        "QListWidgetItem*",
        "itemClickedSlot",
        "currentRowChangedSlot"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'contentsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addPlugin'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'removePlugin'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveUpPlugin'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'moveDownPlugin'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'activatePlugin'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'activateAllPlugins'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'deactivateAllPlugins'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'removeAllPlugins'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'loadPresetPlugin'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'directAccessPlugin'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'propertiesPlugin'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editPlugin'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'importPlugins'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'exportPlugins'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addAudioInsertPlugin'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addAudioAuxSendPlugin'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addMidiInsertPlugin'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addMidiAuxSendPlugin'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addMidiControlPlugin'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertPluginOutputs'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertPluginInputs'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiControlAutoConnect'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioOutputs'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioOutputBus'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioOutputBusName'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioOutputAutoConnect'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'dropMove'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'dropCopy'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'dropCancel'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'itemDoubleClickedSlot'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(32, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 33, 2 },
        }}),
        // Slot 'itemClickedSlot'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(34, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 33, 2 },
        }}),
        // Slot 'currentRowChangedSlot'
        QtMocHelpers::SlotData<void(int)>(35, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorPluginListView, qt_meta_tag_ZN22qtractorPluginListViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorPluginListView::staticMetaObject = { {
    QMetaObject::SuperData::link<QListWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22qtractorPluginListViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22qtractorPluginListViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22qtractorPluginListViewE_t>.metaTypes,
    nullptr
} };

void qtractorPluginListView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorPluginListView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->contentsChanged(); break;
        case 1: _t->addPlugin(); break;
        case 2: _t->removePlugin(); break;
        case 3: _t->moveUpPlugin(); break;
        case 4: _t->moveDownPlugin(); break;
        case 5: _t->activatePlugin(); break;
        case 6: _t->activateAllPlugins(); break;
        case 7: _t->deactivateAllPlugins(); break;
        case 8: _t->removeAllPlugins(); break;
        case 9: _t->loadPresetPlugin(); break;
        case 10: _t->directAccessPlugin(); break;
        case 11: _t->propertiesPlugin(); break;
        case 12: _t->editPlugin(); break;
        case 13: _t->importPlugins(); break;
        case 14: _t->exportPlugins(); break;
        case 15: _t->addAudioInsertPlugin(); break;
        case 16: _t->addAudioAuxSendPlugin(); break;
        case 17: _t->addMidiInsertPlugin(); break;
        case 18: _t->addMidiAuxSendPlugin(); break;
        case 19: _t->addMidiControlPlugin(); break;
        case 20: _t->insertPluginOutputs(); break;
        case 21: _t->insertPluginInputs(); break;
        case 22: _t->midiControlAutoConnect(); break;
        case 23: _t->audioOutputs(); break;
        case 24: _t->audioOutputBus(); break;
        case 25: _t->audioOutputBusName(); break;
        case 26: _t->audioOutputAutoConnect(); break;
        case 27: _t->dropMove(); break;
        case 28: _t->dropCopy(); break;
        case 29: _t->dropCancel(); break;
        case 30: _t->itemDoubleClickedSlot((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 31: _t->itemClickedSlot((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 32: _t->currentRowChangedSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qtractorPluginListView::*)()>(_a, &qtractorPluginListView::contentsChanged, 0))
            return;
    }
}

const QMetaObject *qtractorPluginListView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorPluginListView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22qtractorPluginListViewE_t>.strings))
        return static_cast<void*>(this);
    return QListWidget::qt_metacast(_clname);
}

int qtractorPluginListView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 33;
    }
    return _id;
}

// SIGNAL 0
void qtractorPluginListView::contentsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
