/*
qtractor - An Audio/MIDI multi-track sequencer.

   Copyright (C) 2005-2025, rncbc aka Rui Nuno Capela. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 
*/

/********************************************************************************
** Form generated from reading UI file 'qtractorTempoAdjustForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QTRACTORTEMPOADJUSTFORM_H
#define UI_QTRACTORTEMPOADJUSTFORM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include "qtractorSpinBox.h"

QT_BEGIN_NAMESPACE

class Ui_qtractorTempoAdjustForm
{
public:
    QVBoxLayout *MainBoxLayout;
    QGridLayout *GroupBoxLayout;
    QGroupBox *TempoGroupBox;
    QGridLayout *gridLayout;
    QLabel *TempoTextLabel;
    qtractorTempoSpinBox *TempoSpinBox;
    QSpacerItem *spacerItem;
    QPushButton *TempoTapPushButton;
    QSpacerItem *spacerItem1;
    QPushButton *TempoDetectPushButton;
    QPushButton *TempoResetPushButton;
    QSpacerItem *spacerItem2;
    QGroupBox *RangeGroupBox;
    QGridLayout *gridLayout1;
    QLabel *RangeStartTextLabel;
    qtractorTimeSpinBox *RangeStartSpinBox;
    QLabel *RangeLengthTextLabel;
    qtractorTimeSpinBox *RangeLengthSpinBox;
    QLabel *RangeBeatsTextLabel;
    QSpinBox *RangeBeatsSpinBox;
    QPushButton *AdjustPushButton;
    QSpacerItem *spacerItem3;
    QGroupBox *FormatGroupBox;
    QVBoxLayout *vboxLayout;
    QComboBox *FormatComboBox;
    QSpacerItem *spacerItem4;
    QDialogButtonBox *DialogButtonBox;

    void setupUi(QDialog *qtractorTempoAdjustForm)
    {
        if (qtractorTempoAdjustForm->objectName().isEmpty())
            qtractorTempoAdjustForm->setObjectName("qtractorTempoAdjustForm");
        qtractorTempoAdjustForm->resize(480, 240);
        qtractorTempoAdjustForm->setFocusPolicy(Qt::StrongFocus);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("transportMetro")));
        qtractorTempoAdjustForm->setWindowIcon(icon);
        MainBoxLayout = new QVBoxLayout(qtractorTempoAdjustForm);
        MainBoxLayout->setSpacing(8);
        MainBoxLayout->setContentsMargins(8, 8, 8, 8);
        MainBoxLayout->setObjectName("MainBoxLayout");
        GroupBoxLayout = new QGridLayout();
        GroupBoxLayout->setSpacing(4);
        GroupBoxLayout->setObjectName("GroupBoxLayout");
        TempoGroupBox = new QGroupBox(qtractorTempoAdjustForm);
        TempoGroupBox->setObjectName("TempoGroupBox");
        gridLayout = new QGridLayout(TempoGroupBox);
        gridLayout->setSpacing(4);
        gridLayout->setContentsMargins(8, 8, 8, 8);
        gridLayout->setObjectName("gridLayout");
        TempoTextLabel = new QLabel(TempoGroupBox);
        TempoTextLabel->setObjectName("TempoTextLabel");
        TempoTextLabel->setMinimumSize(QSize(60, 0));

        gridLayout->addWidget(TempoTextLabel, 0, 0, 1, 1);

        TempoSpinBox = new qtractorTempoSpinBox(TempoGroupBox);
        TempoSpinBox->setObjectName("TempoSpinBox");
        TempoSpinBox->setMinimumSize(QSize(100, 10));

        gridLayout->addWidget(TempoSpinBox, 0, 1, 1, 1);

        spacerItem = new QSpacerItem(8, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 0, 2, 1, 1);

        TempoTapPushButton = new QPushButton(TempoGroupBox);
        TempoTapPushButton->setObjectName("TempoTapPushButton");

        gridLayout->addWidget(TempoTapPushButton, 0, 3, 1, 1);

        spacerItem1 = new QSpacerItem(8, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem1, 0, 4, 1, 1);

        TempoDetectPushButton = new QPushButton(TempoGroupBox);
        TempoDetectPushButton->setObjectName("TempoDetectPushButton");

        gridLayout->addWidget(TempoDetectPushButton, 1, 1, 1, 1);

        TempoResetPushButton = new QPushButton(TempoGroupBox);
        TempoResetPushButton->setObjectName("TempoResetPushButton");

        gridLayout->addWidget(TempoResetPushButton, 2, 1, 1, 1);

        spacerItem2 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem2, 3, 0, 1, 5);


        GroupBoxLayout->addWidget(TempoGroupBox, 0, 0, 1, 3);

        RangeGroupBox = new QGroupBox(qtractorTempoAdjustForm);
        RangeGroupBox->setObjectName("RangeGroupBox");
        gridLayout1 = new QGridLayout(RangeGroupBox);
        gridLayout1->setSpacing(4);
        gridLayout1->setContentsMargins(8, 8, 8, 8);
        gridLayout1->setObjectName("gridLayout1");
        RangeStartTextLabel = new QLabel(RangeGroupBox);
        RangeStartTextLabel->setObjectName("RangeStartTextLabel");
        RangeStartTextLabel->setMinimumSize(QSize(60, 0));

        gridLayout1->addWidget(RangeStartTextLabel, 0, 0, 1, 1);

        RangeStartSpinBox = new qtractorTimeSpinBox(RangeGroupBox);
        RangeStartSpinBox->setObjectName("RangeStartSpinBox");
        RangeStartSpinBox->setMinimumSize(QSize(120, 0));

        gridLayout1->addWidget(RangeStartSpinBox, 0, 1, 1, 2);

        RangeLengthTextLabel = new QLabel(RangeGroupBox);
        RangeLengthTextLabel->setObjectName("RangeLengthTextLabel");
        RangeLengthTextLabel->setMinimumSize(QSize(60, 0));

        gridLayout1->addWidget(RangeLengthTextLabel, 1, 0, 1, 1);

        RangeLengthSpinBox = new qtractorTimeSpinBox(RangeGroupBox);
        RangeLengthSpinBox->setObjectName("RangeLengthSpinBox");
        RangeLengthSpinBox->setMinimumSize(QSize(120, 0));

        gridLayout1->addWidget(RangeLengthSpinBox, 1, 1, 1, 2);

        RangeBeatsTextLabel = new QLabel(RangeGroupBox);
        RangeBeatsTextLabel->setObjectName("RangeBeatsTextLabel");
        RangeBeatsTextLabel->setMinimumSize(QSize(60, 0));

        gridLayout1->addWidget(RangeBeatsTextLabel, 2, 0, 1, 1);

        RangeBeatsSpinBox = new QSpinBox(RangeGroupBox);
        RangeBeatsSpinBox->setObjectName("RangeBeatsSpinBox");
        RangeBeatsSpinBox->setMinimum(1);
        RangeBeatsSpinBox->setMaximum(65536);

        gridLayout1->addWidget(RangeBeatsSpinBox, 2, 1, 1, 1);

        AdjustPushButton = new QPushButton(RangeGroupBox);
        AdjustPushButton->setObjectName("AdjustPushButton");

        gridLayout1->addWidget(AdjustPushButton, 2, 2, 1, 1);

        spacerItem3 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout1->addItem(spacerItem3, 3, 0, 1, 3);


        GroupBoxLayout->addWidget(RangeGroupBox, 1, 0, 1, 2);

        FormatGroupBox = new QGroupBox(qtractorTempoAdjustForm);
        FormatGroupBox->setObjectName("FormatGroupBox");
        vboxLayout = new QVBoxLayout(FormatGroupBox);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        vboxLayout->setObjectName("vboxLayout");
        FormatComboBox = new QComboBox(FormatGroupBox);
        FormatComboBox->addItem(QString());
        FormatComboBox->addItem(QString());
        FormatComboBox->addItem(QString());
        FormatComboBox->setObjectName("FormatComboBox");

        vboxLayout->addWidget(FormatComboBox);

        spacerItem4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem4);


        GroupBoxLayout->addWidget(FormatGroupBox, 1, 2, 1, 1);


        MainBoxLayout->addLayout(GroupBoxLayout);

        DialogButtonBox = new QDialogButtonBox(qtractorTempoAdjustForm);
        DialogButtonBox->setObjectName("DialogButtonBox");
        DialogButtonBox->setOrientation(Qt::Horizontal);
        DialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        MainBoxLayout->addWidget(DialogButtonBox);

#if QT_CONFIG(shortcut)
        TempoTextLabel->setBuddy(TempoSpinBox);
        RangeStartTextLabel->setBuddy(RangeStartSpinBox);
        RangeLengthTextLabel->setBuddy(RangeLengthSpinBox);
        RangeBeatsTextLabel->setBuddy(RangeLengthSpinBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(TempoSpinBox, TempoTapPushButton);
        QWidget::setTabOrder(TempoTapPushButton, TempoDetectPushButton);
        QWidget::setTabOrder(TempoDetectPushButton, TempoResetPushButton);
        QWidget::setTabOrder(TempoResetPushButton, RangeStartSpinBox);
        QWidget::setTabOrder(RangeStartSpinBox, RangeLengthSpinBox);
        QWidget::setTabOrder(RangeLengthSpinBox, RangeBeatsSpinBox);
        QWidget::setTabOrder(RangeBeatsSpinBox, AdjustPushButton);
        QWidget::setTabOrder(AdjustPushButton, FormatComboBox);
        QWidget::setTabOrder(FormatComboBox, DialogButtonBox);

        retranslateUi(qtractorTempoAdjustForm);

        QMetaObject::connectSlotsByName(qtractorTempoAdjustForm);
    } // setupUi

    void retranslateUi(QDialog *qtractorTempoAdjustForm)
    {
        qtractorTempoAdjustForm->setWindowTitle(QCoreApplication::translate("qtractorTempoAdjustForm", "Tempo Adjust", nullptr));
        TempoGroupBox->setTitle(QCoreApplication::translate("qtractorTempoAdjustForm", "Metronome", nullptr));
        TempoTextLabel->setText(QCoreApplication::translate("qtractorTempoAdjustForm", "&Tempo:", nullptr));
#if QT_CONFIG(tooltip)
        TempoSpinBox->setToolTip(QCoreApplication::translate("qtractorTempoAdjustForm", "Tempo/Time signature", nullptr));
#endif // QT_CONFIG(tooltip)
        TempoTapPushButton->setText(QCoreApplication::translate("qtractorTempoAdjustForm", "T&ap", nullptr));
        TempoDetectPushButton->setText(QCoreApplication::translate("qtractorTempoAdjustForm", "&Detect", nullptr));
        TempoResetPushButton->setText(QCoreApplication::translate("qtractorTempoAdjustForm", "R&eset", nullptr));
        RangeGroupBox->setTitle(QCoreApplication::translate("qtractorTempoAdjustForm", "Range", nullptr));
        RangeStartTextLabel->setText(QCoreApplication::translate("qtractorTempoAdjustForm", "&Start:", nullptr));
#if QT_CONFIG(tooltip)
        RangeStartSpinBox->setToolTip(QCoreApplication::translate("qtractorTempoAdjustForm", "Range start", nullptr));
#endif // QT_CONFIG(tooltip)
        RangeLengthTextLabel->setText(QCoreApplication::translate("qtractorTempoAdjustForm", "&Length:", nullptr));
#if QT_CONFIG(tooltip)
        RangeLengthSpinBox->setToolTip(QCoreApplication::translate("qtractorTempoAdjustForm", "Range length", nullptr));
#endif // QT_CONFIG(tooltip)
        RangeBeatsTextLabel->setText(QCoreApplication::translate("qtractorTempoAdjustForm", "&Beats:", nullptr));
#if QT_CONFIG(tooltip)
        RangeBeatsSpinBox->setToolTip(QCoreApplication::translate("qtractorTempoAdjustForm", "Range beats", nullptr));
#endif // QT_CONFIG(tooltip)
        AdjustPushButton->setText(QCoreApplication::translate("qtractorTempoAdjustForm", "A&djust", nullptr));
        FormatGroupBox->setTitle(QCoreApplication::translate("qtractorTempoAdjustForm", "Format", nullptr));
        FormatComboBox->setItemText(0, QCoreApplication::translate("qtractorTempoAdjustForm", "Frames", nullptr));
        FormatComboBox->setItemText(1, QCoreApplication::translate("qtractorTempoAdjustForm", "Time", nullptr));
        FormatComboBox->setItemText(2, QCoreApplication::translate("qtractorTempoAdjustForm", "BBT", nullptr));

#if QT_CONFIG(tooltip)
        FormatComboBox->setToolTip(QCoreApplication::translate("qtractorTempoAdjustForm", "Time display format", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class qtractorTempoAdjustForm: public Ui_qtractorTempoAdjustForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QTRACTORTEMPOADJUSTFORM_H
