/****************************************************************************
** Meta object code from reading C++ file 'qtractorPluginForm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorPluginForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorPluginForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18qtractorPluginFormE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorPluginForm::qt_create_metaobjectdata<qt_meta_tag_ZN18qtractorPluginFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorPluginForm",
        "changePresetSlot",
        "",
        "sPreset",
        "loadPresetSlot",
        "iPreset",
        "openPresetSlot",
        "savePresetSlot",
        "deletePresetSlot",
        "aliasSlot",
        "editSlot",
        "bOn",
        "activateSlot",
        "currentChangedSlot",
        "iTab",
        "sendsSlot",
        "returnsSlot",
        "autoConnectSlot",
        "midiControlActionSlot",
        "midiControlMenuSlot",
        "QPoint",
        "pos",
        "changeAuxSendBusNameSlot",
        "iAuxSendBusName",
        "clickAuxSendBusNameSlot",
        "clickAuxSendIOMatrixSlot",
        "updateDirectAccessParamSlot",
        "changeDirectAccessParamSlot",
        "updateParamRangeSlot"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'changePresetSlot'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'loadPresetSlot'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'openPresetSlot'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'savePresetSlot'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'deletePresetSlot'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'aliasSlot'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editSlot'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'activateSlot'
        QtMocHelpers::SlotData<void(bool)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'currentChangedSlot'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'sendsSlot'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'returnsSlot'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'autoConnectSlot'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Slot 'midiControlActionSlot'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiControlMenuSlot'
        QtMocHelpers::SlotData<void(const QPoint &)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'changeAuxSendBusNameSlot'
        QtMocHelpers::SlotData<void(int)>(22, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Slot 'clickAuxSendBusNameSlot'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'clickAuxSendIOMatrixSlot'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateDirectAccessParamSlot'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'changeDirectAccessParamSlot'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateParamRangeSlot'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorPluginForm, qt_meta_tag_ZN18qtractorPluginFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorPluginForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qtractorPluginFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qtractorPluginFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18qtractorPluginFormE_t>.metaTypes,
    nullptr
} };

void qtractorPluginForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorPluginForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changePresetSlot((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->loadPresetSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->openPresetSlot(); break;
        case 3: _t->savePresetSlot(); break;
        case 4: _t->deletePresetSlot(); break;
        case 5: _t->aliasSlot(); break;
        case 6: _t->editSlot((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->activateSlot((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->currentChangedSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: _t->sendsSlot(); break;
        case 10: _t->returnsSlot(); break;
        case 11: _t->autoConnectSlot((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->midiControlActionSlot(); break;
        case 13: _t->midiControlMenuSlot((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 14: _t->changeAuxSendBusNameSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->clickAuxSendBusNameSlot(); break;
        case 16: _t->clickAuxSendIOMatrixSlot(); break;
        case 17: _t->updateDirectAccessParamSlot(); break;
        case 18: _t->changeDirectAccessParamSlot(); break;
        case 19: _t->updateParamRangeSlot(); break;
        default: ;
        }
    }
}

const QMetaObject *qtractorPluginForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorPluginForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18qtractorPluginFormE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int qtractorPluginForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 20;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN25qtractorPluginParamWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorPluginParamWidget::qt_create_metaobjectdata<qt_meta_tag_ZN25qtractorPluginParamWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorPluginParamWidget",
        "updateValue",
        "",
        "fValue",
        "propertyChanged",
        "toolButtonClicked",
        "curveButtonClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'updateValue'
        QtMocHelpers::SlotData<void(float)>(1, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Float, 3 },
        }}),
        // Slot 'propertyChanged'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toolButtonClicked'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'curveButtonClicked'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorPluginParamWidget, qt_meta_tag_ZN25qtractorPluginParamWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorPluginParamWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25qtractorPluginParamWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25qtractorPluginParamWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25qtractorPluginParamWidgetE_t>.metaTypes,
    nullptr
} };

void qtractorPluginParamWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorPluginParamWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateValue((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 1: _t->propertyChanged(); break;
        case 2: _t->toolButtonClicked(); break;
        case 3: _t->curveButtonClicked(); break;
        default: ;
        }
    }
}

const QMetaObject *qtractorPluginParamWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorPluginParamWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25qtractorPluginParamWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int qtractorPluginParamWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
QT_WARNING_POP
