/*
qtractor - An Audio/MIDI multi-track sequencer.

   Copyright (C) 2005-2025, rncbc aka Rui Nuno Capela. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 
*/

/********************************************************************************
** Form generated from reading UI file 'qtractorTrackForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QTRACTORTRACKFORM_H
#define UI_QTRACTORTRACKFORM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "qtractorPluginListView.h"

QT_BEGIN_NAMESPACE

class Ui_qtractorTrackForm
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *TrackTabWidget;
    QWidget *TrackTab;
    QVBoxLayout *vboxLayout1;
    QGridLayout *gridLayout;
    QLabel *TrackNameTextLabel;
    QTextEdit *TrackNameTextEdit;
    QToolButton *TrackIconToolButton;
    QSpacerItem *spacerItem;
    QHBoxLayout *hboxLayout;
    QGroupBox *TrackTypeGroup;
    QHBoxLayout *hboxLayout1;
    QRadioButton *AudioRadioButton;
    QRadioButton *MidiRadioButton;
    QGroupBox *TrackBusGroup;
    QHBoxLayout *hboxLayout2;
    QComboBox *InputBusNameComboBox;
    QComboBox *OutputBusNameComboBox;
    QToolButton *BusNameToolButton;
    QGroupBox *MidiGroupBox;
    QGridLayout *gridLayout1;
    QCheckBox *OmniCheckBox;
    QLabel *ChannelTextLabel;
    QSpinBox *ChannelSpinBox;
    QComboBox *InstrumentComboBox;
    QLabel *BankSelMethodTextLabel;
    QComboBox *BankSelMethodComboBox;
    QCheckBox *DrumsCheckBox;
    QLabel *BankTextLabel;
    QComboBox *BankComboBox;
    QLabel *ProgTextLabel;
    QComboBox *ProgComboBox;
    QGroupBox *TrackViewGroup;
    QHBoxLayout *hboxLayout3;
    QLabel *ForegroundColorTextLabel;
    QComboBox *ForegroundColorComboBox;
    QToolButton *ForegroundColorToolButton;
    QSpacerItem *spacerItem1;
    QLabel *BackgroundColorTextLabel;
    QComboBox *BackgroundColorComboBox;
    QToolButton *BackgroundColorToolButton;
    QCheckBox *AutoBackgroundColorCheckBox;
    QWidget *PluginsTab;
    QGridLayout *gridLayout2;
    qtractorPluginListView *PluginListView;
    QVBoxLayout *vboxLayout2;
    QToolButton *AddPluginToolButton;
    QToolButton *RemovePluginToolButton;
    QSpacerItem *spacerItem2;
    QToolButton *MoveUpPluginToolButton;
    QToolButton *MoveDownPluginToolButton;
    QCheckBox *PluginListLatencyCheckBox;
    QHBoxLayout *hboxLayout4;
    QSpacerItem *spacerItem3;
    QPushButton *PluginListLatencyPushButton;
    QDialogButtonBox *DialogButtonBox;

    void setupUi(QDialog *qtractorTrackForm)
    {
        if (qtractorTrackForm->objectName().isEmpty())
            qtractorTrackForm->setObjectName("qtractorTrackForm");
        qtractorTrackForm->resize(380, 320);
        qtractorTrackForm->setFocusPolicy(Qt::StrongFocus);
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("trackProperties")));
        qtractorTrackForm->setWindowIcon(icon);
        vboxLayout = new QVBoxLayout(qtractorTrackForm);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        vboxLayout->setObjectName("vboxLayout");
        TrackTabWidget = new QTabWidget(qtractorTrackForm);
        TrackTabWidget->setObjectName("TrackTabWidget");
        TrackTab = new QWidget();
        TrackTab->setObjectName("TrackTab");
        vboxLayout1 = new QVBoxLayout(TrackTab);
        vboxLayout1->setSpacing(4);
        vboxLayout1->setContentsMargins(8, 8, 8, 8);
        vboxLayout1->setObjectName("vboxLayout1");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(4);
        gridLayout->setObjectName("gridLayout");
        TrackNameTextLabel = new QLabel(TrackTab);
        TrackNameTextLabel->setObjectName("TrackNameTextLabel");

        gridLayout->addWidget(TrackNameTextLabel, 0, 0, 1, 6);

        TrackNameTextEdit = new QTextEdit(TrackTab);
        TrackNameTextEdit->setObjectName("TrackNameTextEdit");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Ignored);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TrackNameTextEdit->sizePolicy().hasHeightForWidth());
        TrackNameTextEdit->setSizePolicy(sizePolicy);
        TrackNameTextEdit->setMinimumSize(QSize(280, 48));

        gridLayout->addWidget(TrackNameTextEdit, 1, 0, 2, 5);

        TrackIconToolButton = new QToolButton(TrackTab);
        TrackIconToolButton->setObjectName("TrackIconToolButton");
        TrackIconToolButton->setMinimumSize(QSize(48, 48));
        TrackIconToolButton->setMaximumSize(QSize(48, 48));
        TrackIconToolButton->setFocusPolicy(Qt::TabFocus);

        gridLayout->addWidget(TrackIconToolButton, 1, 5, 1, 1);

        spacerItem = new QSpacerItem(8, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 2, 5, 1, 1);


        vboxLayout1->addLayout(gridLayout);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(4);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        TrackTypeGroup = new QGroupBox(TrackTab);
        TrackTypeGroup->setObjectName("TrackTypeGroup");
        hboxLayout1 = new QHBoxLayout(TrackTypeGroup);
        hboxLayout1->setSpacing(4);
        hboxLayout1->setContentsMargins(8, 8, 8, 8);
        hboxLayout1->setObjectName("hboxLayout1");
        AudioRadioButton = new QRadioButton(TrackTypeGroup);
        AudioRadioButton->setObjectName("AudioRadioButton");

        hboxLayout1->addWidget(AudioRadioButton);

        MidiRadioButton = new QRadioButton(TrackTypeGroup);
        MidiRadioButton->setObjectName("MidiRadioButton");

        hboxLayout1->addWidget(MidiRadioButton);


        hboxLayout->addWidget(TrackTypeGroup);

        TrackBusGroup = new QGroupBox(TrackTab);
        TrackBusGroup->setObjectName("TrackBusGroup");
        hboxLayout2 = new QHBoxLayout(TrackBusGroup);
        hboxLayout2->setSpacing(4);
        hboxLayout2->setContentsMargins(8, 8, 8, 8);
        hboxLayout2->setObjectName("hboxLayout2");
        InputBusNameComboBox = new QComboBox(TrackBusGroup);
        InputBusNameComboBox->setObjectName("InputBusNameComboBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(InputBusNameComboBox->sizePolicy().hasHeightForWidth());
        InputBusNameComboBox->setSizePolicy(sizePolicy1);
        InputBusNameComboBox->setMinimumSize(QSize(80, 0));

        hboxLayout2->addWidget(InputBusNameComboBox);

        OutputBusNameComboBox = new QComboBox(TrackBusGroup);
        OutputBusNameComboBox->setObjectName("OutputBusNameComboBox");
        sizePolicy1.setHeightForWidth(OutputBusNameComboBox->sizePolicy().hasHeightForWidth());
        OutputBusNameComboBox->setSizePolicy(sizePolicy1);
        OutputBusNameComboBox->setMinimumSize(QSize(80, 0));

        hboxLayout2->addWidget(OutputBusNameComboBox);

        BusNameToolButton = new QToolButton(TrackBusGroup);
        BusNameToolButton->setObjectName("BusNameToolButton");
        BusNameToolButton->setMinimumSize(QSize(22, 22));
        BusNameToolButton->setMaximumSize(QSize(24, 24));
        BusNameToolButton->setFocusPolicy(Qt::TabFocus);

        hboxLayout2->addWidget(BusNameToolButton);


        hboxLayout->addWidget(TrackBusGroup);


        vboxLayout1->addLayout(hboxLayout);

        MidiGroupBox = new QGroupBox(TrackTab);
        MidiGroupBox->setObjectName("MidiGroupBox");
        gridLayout1 = new QGridLayout(MidiGroupBox);
        gridLayout1->setSpacing(4);
        gridLayout1->setContentsMargins(8, 8, 8, 8);
        gridLayout1->setObjectName("gridLayout1");
        OmniCheckBox = new QCheckBox(MidiGroupBox);
        OmniCheckBox->setObjectName("OmniCheckBox");

        gridLayout1->addWidget(OmniCheckBox, 0, 0, 1, 1);

        ChannelTextLabel = new QLabel(MidiGroupBox);
        ChannelTextLabel->setObjectName("ChannelTextLabel");
        ChannelTextLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

        gridLayout1->addWidget(ChannelTextLabel, 0, 1, 1, 1);

        ChannelSpinBox = new QSpinBox(MidiGroupBox);
        ChannelSpinBox->setObjectName("ChannelSpinBox");
        ChannelSpinBox->setMaximumSize(QSize(48, 24));
        ChannelSpinBox->setMinimum(1);
        ChannelSpinBox->setMaximum(16);

        gridLayout1->addWidget(ChannelSpinBox, 0, 2, 1, 1);

        InstrumentComboBox = new QComboBox(MidiGroupBox);
        InstrumentComboBox->setObjectName("InstrumentComboBox");

        gridLayout1->addWidget(InstrumentComboBox, 0, 3, 1, 2);

        BankSelMethodTextLabel = new QLabel(MidiGroupBox);
        BankSelMethodTextLabel->setObjectName("BankSelMethodTextLabel");
        BankSelMethodTextLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

        gridLayout1->addWidget(BankSelMethodTextLabel, 1, 0, 1, 3);

        BankSelMethodComboBox = new QComboBox(MidiGroupBox);
        BankSelMethodComboBox->setObjectName("BankSelMethodComboBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(BankSelMethodComboBox->sizePolicy().hasHeightForWidth());
        BankSelMethodComboBox->setSizePolicy(sizePolicy2);

        gridLayout1->addWidget(BankSelMethodComboBox, 1, 3, 1, 1);

        DrumsCheckBox = new QCheckBox(MidiGroupBox);
        DrumsCheckBox->setObjectName("DrumsCheckBox");

        gridLayout1->addWidget(DrumsCheckBox, 1, 4, 1, 1);

        BankTextLabel = new QLabel(MidiGroupBox);
        BankTextLabel->setObjectName("BankTextLabel");

        gridLayout1->addWidget(BankTextLabel, 2, 0, 1, 1);

        BankComboBox = new QComboBox(MidiGroupBox);
        BankComboBox->setObjectName("BankComboBox");
        BankComboBox->setEditable(true);

        gridLayout1->addWidget(BankComboBox, 2, 1, 1, 4);

        ProgTextLabel = new QLabel(MidiGroupBox);
        ProgTextLabel->setObjectName("ProgTextLabel");

        gridLayout1->addWidget(ProgTextLabel, 3, 0, 1, 1);

        ProgComboBox = new QComboBox(MidiGroupBox);
        ProgComboBox->setObjectName("ProgComboBox");
        ProgComboBox->setEditable(true);

        gridLayout1->addWidget(ProgComboBox, 3, 1, 1, 4);


        vboxLayout1->addWidget(MidiGroupBox);

        TrackViewGroup = new QGroupBox(TrackTab);
        TrackViewGroup->setObjectName("TrackViewGroup");
        hboxLayout3 = new QHBoxLayout(TrackViewGroup);
        hboxLayout3->setSpacing(4);
        hboxLayout3->setContentsMargins(8, 8, 8, 8);
        hboxLayout3->setObjectName("hboxLayout3");
        ForegroundColorTextLabel = new QLabel(TrackViewGroup);
        ForegroundColorTextLabel->setObjectName("ForegroundColorTextLabel");

        hboxLayout3->addWidget(ForegroundColorTextLabel);

        ForegroundColorComboBox = new QComboBox(TrackViewGroup);
        ForegroundColorComboBox->setObjectName("ForegroundColorComboBox");
        ForegroundColorComboBox->setMinimumSize(QSize(64, 0));
        ForegroundColorComboBox->setEditable(true);

        hboxLayout3->addWidget(ForegroundColorComboBox);

        ForegroundColorToolButton = new QToolButton(TrackViewGroup);
        ForegroundColorToolButton->setObjectName("ForegroundColorToolButton");
        ForegroundColorToolButton->setMinimumSize(QSize(22, 22));
        ForegroundColorToolButton->setMaximumSize(QSize(24, 24));
        ForegroundColorToolButton->setFocusPolicy(Qt::TabFocus);
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("formColor")));
        ForegroundColorToolButton->setIcon(icon1);

        hboxLayout3->addWidget(ForegroundColorToolButton);

        spacerItem1 = new QSpacerItem(8, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout3->addItem(spacerItem1);

        BackgroundColorTextLabel = new QLabel(TrackViewGroup);
        BackgroundColorTextLabel->setObjectName("BackgroundColorTextLabel");
        BackgroundColorTextLabel->setAlignment(Qt::AlignRight|Qt::AlignVCenter);

        hboxLayout3->addWidget(BackgroundColorTextLabel);

        BackgroundColorComboBox = new QComboBox(TrackViewGroup);
        BackgroundColorComboBox->setObjectName("BackgroundColorComboBox");
        BackgroundColorComboBox->setMinimumSize(QSize(64, 0));
        BackgroundColorComboBox->setEditable(true);

        hboxLayout3->addWidget(BackgroundColorComboBox);

        BackgroundColorToolButton = new QToolButton(TrackViewGroup);
        BackgroundColorToolButton->setObjectName("BackgroundColorToolButton");
        BackgroundColorToolButton->setMinimumSize(QSize(22, 22));
        BackgroundColorToolButton->setMaximumSize(QSize(24, 24));
        BackgroundColorToolButton->setFocusPolicy(Qt::TabFocus);
        BackgroundColorToolButton->setIcon(icon1);

        hboxLayout3->addWidget(BackgroundColorToolButton);

        AutoBackgroundColorCheckBox = new QCheckBox(TrackViewGroup);
        AutoBackgroundColorCheckBox->setObjectName("AutoBackgroundColorCheckBox");
        AutoBackgroundColorCheckBox->setChecked(false);

        hboxLayout3->addWidget(AutoBackgroundColorCheckBox);


        vboxLayout1->addWidget(TrackViewGroup);

        TrackTabWidget->addTab(TrackTab, QString());
        PluginsTab = new QWidget();
        PluginsTab->setObjectName("PluginsTab");
        gridLayout2 = new QGridLayout(PluginsTab);
        gridLayout2->setSpacing(4);
        gridLayout2->setContentsMargins(8, 8, 8, 8);
        gridLayout2->setObjectName("gridLayout2");
        PluginListView = new qtractorPluginListView(PluginsTab);
        PluginListView->setObjectName("PluginListView");
        PluginListView->setFocusPolicy(Qt::WheelFocus);

        gridLayout2->addWidget(PluginListView, 0, 0, 1, 2);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(4);
        vboxLayout2->setObjectName("vboxLayout2");
        AddPluginToolButton = new QToolButton(PluginsTab);
        AddPluginToolButton->setObjectName("AddPluginToolButton");
        AddPluginToolButton->setMinimumSize(QSize(90, 28));
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("formCreate")));
        AddPluginToolButton->setIcon(icon2);
        AddPluginToolButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        vboxLayout2->addWidget(AddPluginToolButton);

        RemovePluginToolButton = new QToolButton(PluginsTab);
        RemovePluginToolButton->setObjectName("RemovePluginToolButton");
        RemovePluginToolButton->setMinimumSize(QSize(90, 28));
        QIcon icon3(QIcon::fromTheme(QString::fromUtf8("formRemove")));
        RemovePluginToolButton->setIcon(icon3);
        RemovePluginToolButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        vboxLayout2->addWidget(RemovePluginToolButton);

        spacerItem2 = new QSpacerItem(8, 8, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout2->addItem(spacerItem2);

        MoveUpPluginToolButton = new QToolButton(PluginsTab);
        MoveUpPluginToolButton->setObjectName("MoveUpPluginToolButton");
        MoveUpPluginToolButton->setMinimumSize(QSize(90, 28));
        QIcon icon4(QIcon::fromTheme(QString::fromUtf8("formMoveUp")));
        MoveUpPluginToolButton->setIcon(icon4);
        MoveUpPluginToolButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        vboxLayout2->addWidget(MoveUpPluginToolButton);

        MoveDownPluginToolButton = new QToolButton(PluginsTab);
        MoveDownPluginToolButton->setObjectName("MoveDownPluginToolButton");
        MoveDownPluginToolButton->setMinimumSize(QSize(90, 28));
        QIcon icon5(QIcon::fromTheme(QString::fromUtf8("formMoveDown")));
        MoveDownPluginToolButton->setIcon(icon5);
        MoveDownPluginToolButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        vboxLayout2->addWidget(MoveDownPluginToolButton);


        gridLayout2->addLayout(vboxLayout2, 0, 2, 1, 1);

        PluginListLatencyCheckBox = new QCheckBox(PluginsTab);
        PluginListLatencyCheckBox->setObjectName("PluginListLatencyCheckBox");

        gridLayout2->addWidget(PluginListLatencyCheckBox, 1, 0, 1, 1);

        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setSpacing(0);
        hboxLayout4->setContentsMargins(0, 0, 0, 0);
        hboxLayout4->setObjectName("hboxLayout4");
        spacerItem3 = new QSpacerItem(8, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout4->addItem(spacerItem3);

        PluginListLatencyPushButton = new QPushButton(PluginsTab);
        PluginListLatencyPushButton->setObjectName("PluginListLatencyPushButton");
        PluginListLatencyPushButton->setFlat(true);

        hboxLayout4->addWidget(PluginListLatencyPushButton);


        gridLayout2->addLayout(hboxLayout4, 1, 1, 1, 2);

        TrackTabWidget->addTab(PluginsTab, QString());

        vboxLayout->addWidget(TrackTabWidget);

        DialogButtonBox = new QDialogButtonBox(qtractorTrackForm);
        DialogButtonBox->setObjectName("DialogButtonBox");
        DialogButtonBox->setOrientation(Qt::Horizontal);
        DialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(DialogButtonBox);

#if QT_CONFIG(shortcut)
        TrackNameTextLabel->setBuddy(TrackNameTextEdit);
        ChannelTextLabel->setBuddy(ChannelSpinBox);
        BankSelMethodTextLabel->setBuddy(BankSelMethodComboBox);
        BankTextLabel->setBuddy(BankComboBox);
        ProgTextLabel->setBuddy(ProgComboBox);
        ForegroundColorTextLabel->setBuddy(ForegroundColorComboBox);
        BackgroundColorTextLabel->setBuddy(BackgroundColorComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(TrackNameTextEdit, TrackIconToolButton);
        QWidget::setTabOrder(TrackIconToolButton, AudioRadioButton);
        QWidget::setTabOrder(AudioRadioButton, MidiRadioButton);
        QWidget::setTabOrder(MidiRadioButton, InputBusNameComboBox);
        QWidget::setTabOrder(InputBusNameComboBox, OutputBusNameComboBox);
        QWidget::setTabOrder(OutputBusNameComboBox, BusNameToolButton);
        QWidget::setTabOrder(BusNameToolButton, OmniCheckBox);
        QWidget::setTabOrder(OmniCheckBox, ChannelSpinBox);
        QWidget::setTabOrder(ChannelSpinBox, InstrumentComboBox);
        QWidget::setTabOrder(InstrumentComboBox, BankSelMethodComboBox);
        QWidget::setTabOrder(BankSelMethodComboBox, DrumsCheckBox);
        QWidget::setTabOrder(DrumsCheckBox, BankComboBox);
        QWidget::setTabOrder(BankComboBox, ProgComboBox);
        QWidget::setTabOrder(ProgComboBox, ForegroundColorComboBox);
        QWidget::setTabOrder(ForegroundColorComboBox, ForegroundColorToolButton);
        QWidget::setTabOrder(ForegroundColorToolButton, BackgroundColorComboBox);
        QWidget::setTabOrder(BackgroundColorComboBox, BackgroundColorToolButton);
        QWidget::setTabOrder(BackgroundColorToolButton, AutoBackgroundColorCheckBox);
        QWidget::setTabOrder(AutoBackgroundColorCheckBox, PluginListView);
        QWidget::setTabOrder(PluginListView, AddPluginToolButton);
        QWidget::setTabOrder(AddPluginToolButton, RemovePluginToolButton);
        QWidget::setTabOrder(RemovePluginToolButton, MoveUpPluginToolButton);
        QWidget::setTabOrder(MoveUpPluginToolButton, MoveDownPluginToolButton);
        QWidget::setTabOrder(MoveDownPluginToolButton, PluginListLatencyCheckBox);
        QWidget::setTabOrder(PluginListLatencyCheckBox, DialogButtonBox);

        retranslateUi(qtractorTrackForm);

        TrackTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(qtractorTrackForm);
    } // setupUi

    void retranslateUi(QDialog *qtractorTrackForm)
    {
        qtractorTrackForm->setWindowTitle(QCoreApplication::translate("qtractorTrackForm", "Track", nullptr));
        TrackNameTextLabel->setText(QCoreApplication::translate("qtractorTrackForm", "&Name:", nullptr));
#if QT_CONFIG(tooltip)
        TrackNameTextEdit->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Track name description", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        TrackIconToolButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Track icon", nullptr));
#endif // QT_CONFIG(tooltip)
        TrackTypeGroup->setTitle(QCoreApplication::translate("qtractorTrackForm", "Type", nullptr));
#if QT_CONFIG(tooltip)
        AudioRadioButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Audio track type", nullptr));
#endif // QT_CONFIG(tooltip)
        AudioRadioButton->setText(QCoreApplication::translate("qtractorTrackForm", "&Audio", nullptr));
#if QT_CONFIG(tooltip)
        MidiRadioButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "MIDI track type", nullptr));
#endif // QT_CONFIG(tooltip)
        MidiRadioButton->setText(QCoreApplication::translate("qtractorTrackForm", "&MIDI", nullptr));
        TrackBusGroup->setTitle(QCoreApplication::translate("qtractorTrackForm", "Input / Output", nullptr));
#if QT_CONFIG(tooltip)
        InputBusNameComboBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Input bus name", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        OutputBusNameComboBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Output bus name", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        BusNameToolButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Manage buses", nullptr));
#endif // QT_CONFIG(tooltip)
        BusNameToolButton->setText(QCoreApplication::translate("qtractorTrackForm", "...", nullptr));
        MidiGroupBox->setTitle(QCoreApplication::translate("qtractorTrackForm", "MIDI / Instrument", nullptr));
#if QT_CONFIG(tooltip)
        OmniCheckBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "MIDI Omni: Capture All Channels", nullptr));
#endif // QT_CONFIG(tooltip)
        OmniCheckBox->setText(QCoreApplication::translate("qtractorTrackForm", "&Omni", nullptr));
        ChannelTextLabel->setText(QCoreApplication::translate("qtractorTrackForm", "&Channel:", nullptr));
#if QT_CONFIG(tooltip)
        ChannelSpinBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "MIDI Channel (1-16)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        InstrumentComboBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "MIDI Patch: Instrument", nullptr));
#endif // QT_CONFIG(tooltip)
        BankSelMethodTextLabel->setText(QCoreApplication::translate("qtractorTrackForm", "Bank &Select Method:", nullptr));
#if QT_CONFIG(tooltip)
        BankSelMethodComboBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "MIDI Patch: Bank Select Method", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        DrumsCheckBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "MIDI Patch: Drum Mode", nullptr));
#endif // QT_CONFIG(tooltip)
        DrumsCheckBox->setText(QCoreApplication::translate("qtractorTrackForm", "&Drums", nullptr));
        BankTextLabel->setText(QCoreApplication::translate("qtractorTrackForm", "&Bank:", nullptr));
#if QT_CONFIG(tooltip)
        BankComboBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "MIDI Patch: Bank", nullptr));
#endif // QT_CONFIG(tooltip)
        ProgTextLabel->setText(QCoreApplication::translate("qtractorTrackForm", "&Program:", nullptr));
#if QT_CONFIG(tooltip)
        ProgComboBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "MIDI Patch: Program", nullptr));
#endif // QT_CONFIG(tooltip)
        TrackViewGroup->setTitle(QCoreApplication::translate("qtractorTrackForm", "View / Colors", nullptr));
        ForegroundColorTextLabel->setText(QCoreApplication::translate("qtractorTrackForm", "&Foreground:", nullptr));
#if QT_CONFIG(tooltip)
        ForegroundColorComboBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Foreground color", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        ForegroundColorToolButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Select custom track foreground color", nullptr));
#endif // QT_CONFIG(tooltip)
        ForegroundColorToolButton->setText(QCoreApplication::translate("qtractorTrackForm", "...", nullptr));
        BackgroundColorTextLabel->setText(QCoreApplication::translate("qtractorTrackForm", "Bac&kground:", nullptr));
#if QT_CONFIG(tooltip)
        BackgroundColorComboBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Background color", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        BackgroundColorToolButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Select custom track background color", nullptr));
#endif // QT_CONFIG(tooltip)
        BackgroundColorToolButton->setText(QCoreApplication::translate("qtractorTrackForm", "...", nullptr));
        AutoBackgroundColorCheckBox->setText(QCoreApplication::translate("qtractorTrackForm", "Auto", nullptr));
        TrackTabWidget->setTabText(TrackTabWidget->indexOf(TrackTab), QCoreApplication::translate("qtractorTrackForm", "Track", nullptr));
#if QT_CONFIG(tooltip)
        PluginListView->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Track plugins", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        AddPluginToolButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Add plugin", nullptr));
#endif // QT_CONFIG(tooltip)
        AddPluginToolButton->setText(QCoreApplication::translate("qtractorTrackForm", "&Add...", nullptr));
#if QT_CONFIG(tooltip)
        RemovePluginToolButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Remove plugin", nullptr));
#endif // QT_CONFIG(tooltip)
        RemovePluginToolButton->setText(QCoreApplication::translate("qtractorTrackForm", "&Remove", nullptr));
#if QT_CONFIG(tooltip)
        MoveUpPluginToolButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Move plugin up", nullptr));
#endif // QT_CONFIG(tooltip)
        MoveUpPluginToolButton->setText(QCoreApplication::translate("qtractorTrackForm", "&Up", nullptr));
#if QT_CONFIG(tooltip)
        MoveDownPluginToolButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Move plugin down", nullptr));
#endif // QT_CONFIG(tooltip)
        MoveDownPluginToolButton->setText(QCoreApplication::translate("qtractorTrackForm", "&Down", nullptr));
#if QT_CONFIG(tooltip)
        PluginListLatencyCheckBox->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Whether to enable plugin latency/delay compensation", nullptr));
#endif // QT_CONFIG(tooltip)
        PluginListLatencyCheckBox->setText(QCoreApplication::translate("qtractorTrackForm", "&Latency compensation", nullptr));
#if QT_CONFIG(tooltip)
        PluginListLatencyPushButton->setToolTip(QCoreApplication::translate("qtractorTrackForm", "Current total latency", nullptr));
#endif // QT_CONFIG(tooltip)
        PluginListLatencyPushButton->setText(QString());
        TrackTabWidget->setTabText(TrackTabWidget->indexOf(PluginsTab), QCoreApplication::translate("qtractorTrackForm", "Plugins", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qtractorTrackForm: public Ui_qtractorTrackForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QTRACTORTRACKFORM_H
