/****************************************************************************
** Meta object code from reading C++ file 'qtractorConnectForm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorConnectForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorConnectForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19qtractorConnectFormE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorConnectForm::qt_create_metaobjectdata<qt_meta_tag_ZN19qtractorConnectFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorConnectForm",
        "connectChanged",
        "",
        "audioRefresh",
        "audioClear",
        "midiRefresh",
        "midiClear",
        "audioReset",
        "midiReset",
        "audioIClientChanged",
        "audioOClientChanged",
        "audioConnectSelected",
        "audioDisconnectSelected",
        "audioDisconnectAll",
        "audioConnectChanged",
        "audioStabilize",
        "midiIClientChanged",
        "midiOClientChanged",
        "midiConnectSelected",
        "midiDisconnectSelected",
        "midiDisconnectAll",
        "midiConnectChanged",
        "midiStabilize"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'connectChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'audioRefresh'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'audioClear'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'midiRefresh'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'midiClear'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'audioReset'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'midiReset'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'audioIClientChanged'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioOClientChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioConnectSelected'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioDisconnectSelected'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioDisconnectAll'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioConnectChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'audioStabilize'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiIClientChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiOClientChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiConnectSelected'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiDisconnectSelected'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiDisconnectAll'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiConnectChanged'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'midiStabilize'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorConnectForm, qt_meta_tag_ZN19qtractorConnectFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorConnectForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qtractorConnectFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qtractorConnectFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19qtractorConnectFormE_t>.metaTypes,
    nullptr
} };

void qtractorConnectForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorConnectForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->connectChanged(); break;
        case 1: _t->audioRefresh(); break;
        case 2: _t->audioClear(); break;
        case 3: _t->midiRefresh(); break;
        case 4: _t->midiClear(); break;
        case 5: _t->audioReset(); break;
        case 6: _t->midiReset(); break;
        case 7: _t->audioIClientChanged(); break;
        case 8: _t->audioOClientChanged(); break;
        case 9: _t->audioConnectSelected(); break;
        case 10: _t->audioDisconnectSelected(); break;
        case 11: _t->audioDisconnectAll(); break;
        case 12: _t->audioConnectChanged(); break;
        case 13: _t->audioStabilize(); break;
        case 14: _t->midiIClientChanged(); break;
        case 15: _t->midiOClientChanged(); break;
        case 16: _t->midiConnectSelected(); break;
        case 17: _t->midiDisconnectSelected(); break;
        case 18: _t->midiDisconnectAll(); break;
        case 19: _t->midiConnectChanged(); break;
        case 20: _t->midiStabilize(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qtractorConnectForm::*)()>(_a, &qtractorConnectForm::connectChanged, 0))
            return;
    }
}

const QMetaObject *qtractorConnectForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorConnectForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19qtractorConnectFormE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int qtractorConnectForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void qtractorConnectForm::connectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
