/****************************************************************************
** Meta object code from reading C++ file 'qtractorBusForm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorBusForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorBusForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15qtractorBusFormE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorBusForm::qt_create_metaobjectdata<qt_meta_tag_ZN15qtractorBusFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorBusForm",
        "reject",
        "",
        "selectBus",
        "moveUpBus",
        "moveDownBus",
        "createBus",
        "updateBus",
        "deleteBus",
        "changed",
        "stabilizeForm",
        "contextMenu",
        "QPoint",
        "midiSysex",
        "addInputPlugin",
        "removeInputPlugin",
        "moveUpInputPlugin",
        "moveDownInputPlugin",
        "addOutputPlugin",
        "removeOutputPlugin",
        "moveUpOutputPlugin",
        "moveDownOutputPlugin"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'reject'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'selectBus'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveUpBus'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveDownBus'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'createBus'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateBus'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'deleteBus'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'changed'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'stabilizeForm'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'contextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'midiSysex'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addInputPlugin'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'removeInputPlugin'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveUpInputPlugin'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveDownInputPlugin'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addOutputPlugin'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'removeOutputPlugin'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveUpOutputPlugin'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveDownOutputPlugin'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorBusForm, qt_meta_tag_ZN15qtractorBusFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorBusForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qtractorBusFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qtractorBusFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15qtractorBusFormE_t>.metaTypes,
    nullptr
} };

void qtractorBusForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorBusForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->reject(); break;
        case 1: _t->selectBus(); break;
        case 2: _t->moveUpBus(); break;
        case 3: _t->moveDownBus(); break;
        case 4: _t->createBus(); break;
        case 5: _t->updateBus(); break;
        case 6: _t->deleteBus(); break;
        case 7: _t->changed(); break;
        case 8: _t->stabilizeForm(); break;
        case 9: _t->contextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 10: _t->midiSysex(); break;
        case 11: _t->addInputPlugin(); break;
        case 12: _t->removeInputPlugin(); break;
        case 13: _t->moveUpInputPlugin(); break;
        case 14: _t->moveDownInputPlugin(); break;
        case 15: _t->addOutputPlugin(); break;
        case 16: _t->removeOutputPlugin(); break;
        case 17: _t->moveUpOutputPlugin(); break;
        case 18: _t->moveDownOutputPlugin(); break;
        default: ;
        }
    }
}

const QMetaObject *qtractorBusForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorBusForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15qtractorBusFormE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int qtractorBusForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
