/****************************************************************************
** Meta object code from reading C++ file 'qtractorObserverWidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorObserverWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorObserverWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24qtractorObserverCheckBoxE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorObserverCheckBox::qt_create_metaobjectdata<qt_meta_tag_ZN24qtractorObserverCheckBoxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorObserverCheckBox",
        "valueChanged",
        "",
        "checkBoxChanged",
        "bValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(float)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 2 },
        }}),
        // Slot 'checkBoxChanged'
        QtMocHelpers::SlotData<void(bool)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorObserverCheckBox, qt_meta_tag_ZN24qtractorObserverCheckBoxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorObserverCheckBox::staticMetaObject = { {
    QMetaObject::SuperData::link<qtractorObserverWidget<QCheckBox>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24qtractorObserverCheckBoxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24qtractorObserverCheckBoxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24qtractorObserverCheckBoxE_t>.metaTypes,
    nullptr
} };

void qtractorObserverCheckBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorObserverCheckBox *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 1: _t->checkBoxChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qtractorObserverCheckBox::*)(float )>(_a, &qtractorObserverCheckBox::valueChanged, 0))
            return;
    }
}

const QMetaObject *qtractorObserverCheckBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorObserverCheckBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24qtractorObserverCheckBoxE_t>.strings))
        return static_cast<void*>(this);
    return qtractorObserverWidget<QCheckBox>::qt_metacast(_clname);
}

int qtractorObserverCheckBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qtractorObserverWidget<QCheckBox>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void qtractorObserverCheckBox::valueChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN23qtractorObserverSpinBoxE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorObserverSpinBox::qt_create_metaobjectdata<qt_meta_tag_ZN23qtractorObserverSpinBoxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorObserverSpinBox",
        "valueChanged",
        "",
        "spinBoxChanged",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(float)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 2 },
        }}),
        // Slot 'spinBoxChanged'
        QtMocHelpers::SlotData<void(double)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Double, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorObserverSpinBox, qt_meta_tag_ZN23qtractorObserverSpinBoxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorObserverSpinBox::staticMetaObject = { {
    QMetaObject::SuperData::link<qtractorObserverWidget<qtractorSpinBox>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23qtractorObserverSpinBoxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23qtractorObserverSpinBoxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23qtractorObserverSpinBoxE_t>.metaTypes,
    nullptr
} };

void qtractorObserverSpinBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorObserverSpinBox *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 1: _t->spinBoxChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qtractorObserverSpinBox::*)(float )>(_a, &qtractorObserverSpinBox::valueChanged, 0))
            return;
    }
}

const QMetaObject *qtractorObserverSpinBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorObserverSpinBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23qtractorObserverSpinBoxE_t>.strings))
        return static_cast<void*>(this);
    return qtractorObserverWidget<qtractorSpinBox>::qt_metacast(_clname);
}

int qtractorObserverSpinBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qtractorObserverWidget<qtractorSpinBox>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void qtractorObserverSpinBox::valueChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN22qtractorObserverSliderE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorObserverSlider::qt_create_metaobjectdata<qt_meta_tag_ZN22qtractorObserverSliderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorObserverSlider",
        "valueChanged",
        "",
        "sliderChanged",
        "iValue"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(float)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 2 },
        }}),
        // Slot 'sliderChanged'
        QtMocHelpers::SlotData<void(int)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorObserverSlider, qt_meta_tag_ZN22qtractorObserverSliderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorObserverSlider::staticMetaObject = { {
    QMetaObject::SuperData::link<qtractorObserverWidget<QSlider>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22qtractorObserverSliderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22qtractorObserverSliderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22qtractorObserverSliderE_t>.metaTypes,
    nullptr
} };

void qtractorObserverSlider::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorObserverSlider *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 1: _t->sliderChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (qtractorObserverSlider::*)(float )>(_a, &qtractorObserverSlider::valueChanged, 0))
            return;
    }
}

const QMetaObject *qtractorObserverSlider::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorObserverSlider::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22qtractorObserverSliderE_t>.strings))
        return static_cast<void*>(this);
    return qtractorObserverWidget<QSlider>::qt_metacast(_clname);
}

int qtractorObserverSlider::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = qtractorObserverWidget<QSlider>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void qtractorObserverSlider::valueChanged(float _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
