/****************************************************************************
** Meta object code from reading C++ file 'qtractorMidiEditorForm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorMidiEditorForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorMidiEditorForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN22qtractorMidiEditorFormE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorMidiEditorForm::qt_create_metaobjectdata<qt_meta_tag_ZN22qtractorMidiEditorFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorMidiEditorForm",
        "stabilizeForm",
        "",
        "fileSave",
        "fileSaveAs",
        "fileMute",
        "fileUnlink",
        "fileRecordEx",
        "bOn",
        "fileTrackInputs",
        "fileTrackOutputs",
        "fileTrackProperties",
        "fileProperties",
        "fileRangeSet",
        "fileLoopSet",
        "fileClose",
        "editUndo",
        "editRedo",
        "editCut",
        "editCopy",
        "editPaste",
        "editPasteRepeat",
        "editDelete",
        "editModeOn",
        "editModeOff",
        "editModeDraw",
        "editSelectAll",
        "editSelectNone",
        "editSelectInvert",
        "editSelectRange",
        "editInsertRange",
        "editInsertStep",
        "editRemoveRange",
        "toolsQuantize",
        "toolsTranspose",
        "toolsNormalize",
        "toolsRandomize",
        "toolsResize",
        "toolsRescale",
        "toolsTimeshift",
        "toolsTemporamp",
        "viewMenubar",
        "viewStatusbar",
        "viewToolbarFile",
        "viewToolbarEdit",
        "viewToolbarView",
        "viewToolbarTransport",
        "viewToolbarTime",
        "viewToolbarScale",
        "viewToolbarThumb",
        "viewToolbarLocked",
        "viewEvents",
        "viewNoteNames",
        "viewNoteDuration",
        "viewNoteColor",
        "viewNoteType",
        "viewDrumMode",
        "viewValueColor",
        "viewValueType",
        "viewGhostTrack",
        "viewZoomIn",
        "viewZoomOut",
        "viewZoomReset",
        "viewZoomHorizontal",
        "viewZoomVertical",
        "viewZoomAll",
        "viewSnap",
        "viewScaleKey",
        "viewScaleType",
        "viewSnapZebra",
        "viewSnapGrid",
        "viewToolTips",
        "viewRefresh",
        "viewPreview",
        "viewFollow",
        "transportStepBackward",
        "transportStepForward",
        "transportStepNoteBackward",
        "transportStepNoteForward",
        "helpShortcuts",
        "helpAbout",
        "helpAboutQt",
        "updateNoteTypeMenu",
        "updateValueTypeMenu",
        "updateGhostTrackMenu",
        "updateZoomMenu",
        "updateSnapMenu",
        "updateScaleMenu",
        "updateTrackInstrumentMenu",
        "sendNote",
        "iNote",
        "iVelocity",
        "bForce",
        "sendNoteEx",
        "iDuration",
        "selectionChanged",
        "qtractorMidiEditor*",
        "contentsChanged",
        "viewTypeChanged",
        "eventTypeChanged",
        "eventParamChanged",
        "snapToScaleKeyChanged",
        "iSnapToScaleKey",
        "snapToScaleTypeChanged",
        "iSnapToScaleType",
        "transportTimeFormatChanged",
        "iDisplayFormat",
        "transportTimeChanged",
        "iPlayHead",
        "transportTimeFinished",
        "transportTempoChanged",
        "fTempo",
        "iBeatsPerBar",
        "iBeatDivisor",
        "transportTempoFinished",
        "transportTempoContextMenu",
        "QPoint",
        "pos",
        "timeSig2ResetClicked",
        "snapPerBeatChanged",
        "iSnapPerBeat",
        "posChanged",
        "sizeChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'stabilizeForm'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileSave'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileSaveAs'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileMute'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileUnlink'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileRecordEx'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'fileTrackInputs'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileTrackOutputs'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileTrackProperties'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileProperties'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileRangeSet'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileLoopSet'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fileClose'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editUndo'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editRedo'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editCut'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editCopy'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editPaste'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editPasteRepeat'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editDelete'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editModeOn'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'editModeOff'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'editModeDraw'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'editSelectAll'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editSelectNone'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editSelectInvert'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editSelectRange'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editInsertRange'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editInsertStep'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editRemoveRange'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toolsQuantize'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toolsTranspose'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toolsNormalize'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toolsRandomize'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toolsResize'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toolsRescale'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toolsTimeshift'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toolsTemporamp'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewMenubar'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewStatusbar'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewToolbarFile'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewToolbarEdit'
        QtMocHelpers::SlotData<void(bool)>(44, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewToolbarView'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewToolbarTransport'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewToolbarTime'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewToolbarScale'
        QtMocHelpers::SlotData<void(bool)>(48, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewToolbarThumb'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewToolbarLocked'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewEvents'
        QtMocHelpers::SlotData<void(bool)>(51, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewNoteNames'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewNoteDuration'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewNoteColor'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewNoteType'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewDrumMode'
        QtMocHelpers::SlotData<void(bool)>(56, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewValueColor'
        QtMocHelpers::SlotData<void(bool)>(57, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewValueType'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewGhostTrack'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewZoomIn'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewZoomOut'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewZoomReset'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewZoomHorizontal'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewZoomVertical'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewZoomAll'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewSnap'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewScaleKey'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewScaleType'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewSnapZebra'
        QtMocHelpers::SlotData<void(bool)>(69, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewSnapGrid'
        QtMocHelpers::SlotData<void(bool)>(70, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewToolTips'
        QtMocHelpers::SlotData<void(bool)>(71, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewRefresh'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'viewPreview'
        QtMocHelpers::SlotData<void(bool)>(73, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'viewFollow'
        QtMocHelpers::SlotData<void(bool)>(74, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'transportStepBackward'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'transportStepForward'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'transportStepNoteBackward'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'transportStepNoteForward'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'helpShortcuts'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'helpAbout'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'helpAboutQt'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateNoteTypeMenu'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateValueTypeMenu'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateGhostTrackMenu'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateZoomMenu'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateSnapMenu'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateScaleMenu'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateTrackInstrumentMenu'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'sendNote'
        QtMocHelpers::SlotData<void(int, int, bool)>(89, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 90 }, { QMetaType::Int, 91 }, { QMetaType::Bool, 92 },
        }}),
        // Slot 'sendNoteEx'
        QtMocHelpers::SlotData<void(int, int, unsigned long)>(93, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 90 }, { QMetaType::Int, 91 }, { QMetaType::ULong, 94 },
        }}),
        // Slot 'selectionChanged'
        QtMocHelpers::SlotData<void(qtractorMidiEditor *)>(95, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 96, 2 },
        }}),
        // Slot 'contentsChanged'
        QtMocHelpers::SlotData<void(qtractorMidiEditor *)>(97, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 96, 2 },
        }}),
        // Slot 'viewTypeChanged'
        QtMocHelpers::SlotData<void(int)>(98, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'eventTypeChanged'
        QtMocHelpers::SlotData<void(int)>(99, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'eventParamChanged'
        QtMocHelpers::SlotData<void(int)>(100, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'snapToScaleKeyChanged'
        QtMocHelpers::SlotData<void(int)>(101, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 102 },
        }}),
        // Slot 'snapToScaleTypeChanged'
        QtMocHelpers::SlotData<void(int)>(103, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 104 },
        }}),
        // Slot 'transportTimeFormatChanged'
        QtMocHelpers::SlotData<void(int)>(105, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 106 },
        }}),
        // Slot 'transportTimeChanged'
        QtMocHelpers::SlotData<void(unsigned long)>(107, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::ULong, 108 },
        }}),
        // Slot 'transportTimeFinished'
        QtMocHelpers::SlotData<void()>(109, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'transportTempoChanged'
        QtMocHelpers::SlotData<void(float, unsigned short, unsigned short)>(110, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Float, 111 }, { QMetaType::UShort, 112 }, { QMetaType::UShort, 113 },
        }}),
        // Slot 'transportTempoFinished'
        QtMocHelpers::SlotData<void()>(114, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'transportTempoContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(115, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 116, 117 },
        }}),
        // Slot 'timeSig2ResetClicked'
        QtMocHelpers::SlotData<void()>(118, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'snapPerBeatChanged'
        QtMocHelpers::SlotData<void(int)>(119, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 120 },
        }}),
        // Slot 'posChanged'
        QtMocHelpers::SlotData<void()>(121, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'sizeChanged'
        QtMocHelpers::SlotData<void()>(122, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorMidiEditorForm, qt_meta_tag_ZN22qtractorMidiEditorFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorMidiEditorForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22qtractorMidiEditorFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22qtractorMidiEditorFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22qtractorMidiEditorFormE_t>.metaTypes,
    nullptr
} };

void qtractorMidiEditorForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorMidiEditorForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stabilizeForm(); break;
        case 1: _t->fileSave(); break;
        case 2: _t->fileSaveAs(); break;
        case 3: _t->fileMute(); break;
        case 4: _t->fileUnlink(); break;
        case 5: _t->fileRecordEx((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->fileTrackInputs(); break;
        case 7: _t->fileTrackOutputs(); break;
        case 8: _t->fileTrackProperties(); break;
        case 9: _t->fileProperties(); break;
        case 10: _t->fileRangeSet(); break;
        case 11: _t->fileLoopSet(); break;
        case 12: _t->fileClose(); break;
        case 13: _t->editUndo(); break;
        case 14: _t->editRedo(); break;
        case 15: _t->editCut(); break;
        case 16: _t->editCopy(); break;
        case 17: _t->editPaste(); break;
        case 18: _t->editPasteRepeat(); break;
        case 19: _t->editDelete(); break;
        case 20: _t->editModeOn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->editModeOff((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->editModeDraw((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->editSelectAll(); break;
        case 24: _t->editSelectNone(); break;
        case 25: _t->editSelectInvert(); break;
        case 26: _t->editSelectRange(); break;
        case 27: _t->editInsertRange(); break;
        case 28: _t->editInsertStep(); break;
        case 29: _t->editRemoveRange(); break;
        case 30: _t->toolsQuantize(); break;
        case 31: _t->toolsTranspose(); break;
        case 32: _t->toolsNormalize(); break;
        case 33: _t->toolsRandomize(); break;
        case 34: _t->toolsResize(); break;
        case 35: _t->toolsRescale(); break;
        case 36: _t->toolsTimeshift(); break;
        case 37: _t->toolsTemporamp(); break;
        case 38: _t->viewMenubar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->viewStatusbar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->viewToolbarFile((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->viewToolbarEdit((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: _t->viewToolbarView((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->viewToolbarTransport((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 44: _t->viewToolbarTime((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 45: _t->viewToolbarScale((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->viewToolbarThumb((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 47: _t->viewToolbarLocked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 48: _t->viewEvents((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 49: _t->viewNoteNames((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 50: _t->viewNoteDuration((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 51: _t->viewNoteColor((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 52: _t->viewNoteType(); break;
        case 53: _t->viewDrumMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 54: _t->viewValueColor((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 55: _t->viewValueType(); break;
        case 56: _t->viewGhostTrack(); break;
        case 57: _t->viewZoomIn(); break;
        case 58: _t->viewZoomOut(); break;
        case 59: _t->viewZoomReset(); break;
        case 60: _t->viewZoomHorizontal(); break;
        case 61: _t->viewZoomVertical(); break;
        case 62: _t->viewZoomAll(); break;
        case 63: _t->viewSnap(); break;
        case 64: _t->viewScaleKey(); break;
        case 65: _t->viewScaleType(); break;
        case 66: _t->viewSnapZebra((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 67: _t->viewSnapGrid((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 68: _t->viewToolTips((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 69: _t->viewRefresh(); break;
        case 70: _t->viewPreview((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 71: _t->viewFollow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 72: _t->transportStepBackward(); break;
        case 73: _t->transportStepForward(); break;
        case 74: _t->transportStepNoteBackward(); break;
        case 75: _t->transportStepNoteForward(); break;
        case 76: _t->helpShortcuts(); break;
        case 77: _t->helpAbout(); break;
        case 78: _t->helpAboutQt(); break;
        case 79: _t->updateNoteTypeMenu(); break;
        case 80: _t->updateValueTypeMenu(); break;
        case 81: _t->updateGhostTrackMenu(); break;
        case 82: _t->updateZoomMenu(); break;
        case 83: _t->updateSnapMenu(); break;
        case 84: _t->updateScaleMenu(); break;
        case 85: _t->updateTrackInstrumentMenu(); break;
        case 86: _t->sendNote((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 87: _t->sendNoteEx((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<ulong>>(_a[3]))); break;
        case 88: _t->selectionChanged((*reinterpret_cast<std::add_pointer_t<qtractorMidiEditor*>>(_a[1]))); break;
        case 89: _t->contentsChanged((*reinterpret_cast<std::add_pointer_t<qtractorMidiEditor*>>(_a[1]))); break;
        case 90: _t->viewTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 91: _t->eventTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 92: _t->eventParamChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 93: _t->snapToScaleKeyChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 94: _t->snapToScaleTypeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 95: _t->transportTimeFormatChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 96: _t->transportTimeChanged((*reinterpret_cast<std::add_pointer_t<ulong>>(_a[1]))); break;
        case 97: _t->transportTimeFinished(); break;
        case 98: _t->transportTempoChanged((*reinterpret_cast<std::add_pointer_t<float>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<ushort>>(_a[3]))); break;
        case 99: _t->transportTempoFinished(); break;
        case 100: _t->transportTempoContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 101: _t->timeSig2ResetClicked(); break;
        case 102: _t->snapPerBeatChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 103: _t->posChanged(); break;
        case 104: _t->sizeChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *qtractorMidiEditorForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorMidiEditorForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22qtractorMidiEditorFormE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int qtractorMidiEditorForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 105)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 105;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 105)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 105;
    }
    return _id;
}
QT_WARNING_POP
