/****************************************************************************
** Meta object code from reading C++ file 'qtractorTrackForm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qtractor/src/qtractorTrackForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtractorTrackForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17qtractorTrackFormE_t {};
} // unnamed namespace

template <> constexpr inline auto qtractorTrackForm::qt_create_metaobjectdata<qt_meta_tag_ZN17qtractorTrackFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "qtractorTrackForm",
        "accept",
        "",
        "reject",
        "stabilizeForm",
        "changed",
        "pluginListChanged",
        "trackTypeChanged",
        "inputBusNameChanged",
        "iInputBusName",
        "outputBusNameChanged",
        "iOutputBusName",
        "busNameClicked",
        "channelChanged",
        "iChannel",
        "instrumentChanged",
        "iInstrument",
        "bankSelMethodChanged",
        "iBankSelMethod",
        "bankChanged",
        "progChanged",
        "foregroundColorChanged",
        "sText",
        "backgroundColorChanged",
        "autoBackgroundColorChanged",
        "selectForegroundColor",
        "selectBackgroundColor",
        "addPlugin",
        "removePlugin",
        "moveUpPlugin",
        "moveDownPlugin",
        "updatePluginListLatency",
        "trackIconAction",
        "trackIconClicked",
        "trackIconChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'accept'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'reject'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'stabilizeForm'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'changed'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'pluginListChanged'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'trackTypeChanged'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'inputBusNameChanged'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Slot 'outputBusNameChanged'
        QtMocHelpers::SlotData<void(int)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'busNameClicked'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'channelChanged'
        QtMocHelpers::SlotData<void(int)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Slot 'instrumentChanged'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'bankSelMethodChanged'
        QtMocHelpers::SlotData<void(int)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Slot 'bankChanged'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'progChanged'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'foregroundColorChanged'
        QtMocHelpers::SlotData<void(const QString &)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'backgroundColorChanged'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'autoBackgroundColorChanged'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'selectForegroundColor'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'selectBackgroundColor'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addPlugin'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'removePlugin'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveUpPlugin'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'moveDownPlugin'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updatePluginListLatency'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'trackIconAction'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'trackIconClicked'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'trackIconChanged'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<qtractorTrackForm, qt_meta_tag_ZN17qtractorTrackFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject qtractorTrackForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17qtractorTrackFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17qtractorTrackFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17qtractorTrackFormE_t>.metaTypes,
    nullptr
} };

void qtractorTrackForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<qtractorTrackForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accept(); break;
        case 1: _t->reject(); break;
        case 2: _t->stabilizeForm(); break;
        case 3: _t->changed(); break;
        case 4: _t->pluginListChanged(); break;
        case 5: _t->trackTypeChanged(); break;
        case 6: _t->inputBusNameChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->outputBusNameChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->busNameClicked(); break;
        case 9: _t->channelChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->instrumentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->bankSelMethodChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->bankChanged(); break;
        case 13: _t->progChanged(); break;
        case 14: _t->foregroundColorChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->backgroundColorChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->autoBackgroundColorChanged(); break;
        case 17: _t->selectForegroundColor(); break;
        case 18: _t->selectBackgroundColor(); break;
        case 19: _t->addPlugin(); break;
        case 20: _t->removePlugin(); break;
        case 21: _t->moveUpPlugin(); break;
        case 22: _t->moveDownPlugin(); break;
        case 23: _t->updatePluginListLatency(); break;
        case 24: _t->trackIconAction(); break;
        case 25: _t->trackIconClicked(); break;
        case 26: _t->trackIconChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *qtractorTrackForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *qtractorTrackForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17qtractorTrackFormE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int qtractorTrackForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 27;
    }
    return _id;
}
QT_WARNING_POP
